#! /usr/local/bin/perl -w 
use strict;
use IO::Handle;
use Tie::File;
use Getopt::Long;
use FileHandle;
use English;
use File::Basename;
#use File::Path qw(mkpath rmtree);

########################################################################################################################
#
#Author : Vrunda Sheth
#
#
#Purpose : To make a parser which will parse the afg file and get the read tracking information for the adaptor
#
#
#Usage : perl solid_velvet_postprocessor.pl -afgfile <velvet afg file> -run <type of run>  -csfasta <csfasta files> -output <output file>
#
#
###########################################################################################################################


#options
use vars qw($opt_help);
use vars qw($opt_run);
use vars qw($opt_csfasta);
use vars qw($opt_afgfile);
use vars qw($opt_output);
use vars qw($opt_debug);
use vars qw($opt_version);
use vars qw($PROG         );



my @filenames;
&GetOptions( "help|h",
             "run|r=s",
             "afgfile|f|afg=s",
	     "csfasta|c=s" => \@filenames,
	     "output|o=s",
             "debug",
             "version|v",
           ) or die "Unknown option\n";

$PROG = basename($0);
my $version=1;
$opt_version=$opt_version if ($opt_version);
if ($opt_version)
{
        die "Version is $version.0.0\n";
}

if(!$opt_afgfile && !$opt_csfasta && !$opt_output || $opt_help) 
{

print "\nUsage:$PROG [-help] --afgfile afg file --csfasta csfasta file1 [--csfasta csfasta file2] --run 0 --output outputfile.\n";
print "help        : Print usage.\n";
print "afgfile     : The afg file of velvet read tracking run.\n";
print "csfasta     : All the csfasta file[s] in the same order as input to velvet\n";
print "run         : Input fragment or f for a fragment run and mates or m for a mate pair run\n";
die "output      : File where the output will be stored.\n";
}

# Default run is set to a fragemnt run.

my ($option)=('f');
# global variables
my $infile;
my $outfile;
my $contig_id;
my $read_id;
my $even;
my $off;
my $position;
my @pos;
my $start;
my $end;
my $actual_position;
my @csread;
my $csfasta_file;
my $read;
my @lines;
my $data;
my $final;
my @info;
my $seq;
my $i;
my @file;
my $state=0;

$infile=$opt_afgfile;
open(IN,$infile) or die "Cannot open $infile $!\n";
while(<IN>)
{
	if ( $_ !~ /^\{/ )
	{
		die "$infile is not afg or is not in proper afg format\n";
	}
	else
	{
		last;
	}
}
close(IN);
#$infile=$opt_file;
mkdir("tmp", 0777) || print $!;
open(TEMP1,">tmp/temp1") or die "Cannot write file in temp directory\n";


#concatenate all the csfasta files into 1 file.
foreach  my $files (@filenames) 
{
	open(TEMP2,$files) or die "Cannot open $files $!\n";
	while(<TEMP2>)
	{
		chomp;
		if ( $_ !~ /^#/ )
		{
			print TEMP1 "$_\n";
		}
		
	}
	close(TEMP2);
   
}
close(TEMP1);
	

#mates vs frags run . Enter 0 for Frags and 1 for mates
$option = $opt_run if ($opt_run);

# final outputfile.
$final =$opt_output;
open(IN,$infile) or die "Cannot open $infile $!\n";
open(OUT,">tmp/outfile_temp") or die "Cannot open $outfile $!\n";
open(TEMP1,"tmp/temp1") or die "Cannot open $csfasta_file $!\n";


# parsing the afg file beings here
while(<IN>)
{
        chomp;
	if ($state==0)
	{
        	if( $_ =~ /^\{CTG/)
        	{
                	$state=1;
			redo;
	
        	}
	}
	else
	{
        	if ( $_ =~ /^\{CTG/ )
        	{
			print OUT "-end\n";
        	}
		if (  $_ =~ /^\{RED/ )
		{
			$state=0;
			redo;
		}
		if ($_=~/^\{SCF/)
		{
			$state=0;
			redo;
		}
        	if( $_ =~ /^eid/ )
        	{
             		$contig_id=$_;
             		$contig_id=~s/eid://;
             		print OUT "=NODE_$contig_id\n";
		}
        	if ( $_ =~ /^src/ )
        	{
                	$read_id=$_;
                	$read_id=~s/src://;
                	print OUT "$read_id\t";
        	}
        	if ( $_ =~ /^off/ )
	 	{
                	$off=$_;
                	$off=~s/off://;
        	}
        	if( $_ =~ /^clr/ )
        	{
                	$position=$_;
                	$position=~s/clr://;
                	@pos=split(/,/,$position);
                	$start=$pos[0];
               	 	$end=$pos[1];
			print OUT "$off\t";
			print OUT"$start\t";
			print OUT"$end\n";

        	}

  	}
   
}

close(IN);
close(OUT);
open(OUT,"tmp/outfile_temp") or die "Cannot open $outfile $!\n";
open(FINAL,">$final") or die "Cannot open $final $!\n";
tie  @lines, 'Tie::File', "tmp/temp1" or die $!;

#calculating the actual position and importing colorspace reads from the csfasta file.

while(<OUT>)
{

        chomp;
        if( $_ =~ /^\d/ )
        {
            @info=split(/\s+/,$_);
            if ($info[1] < 0 )
            {

            }
            else
            {
                my $seq_id= $lines[($info[0]+($info[0]-1)-1)];
                $seq= $lines[($info[0]+($info[0]-1))];
		my @individual_length=split(//,$seq);
		my $scalar=$#individual_length-1;
		if ($option eq 'm' || $option eq 'mates'|| $option eq 'M'|| $option eq 'Mates' )
		{	
			if ($info[0] %2==0)
			{
				if( $info[2] < $info[3])
                        	{
                        	       $actual_position=$info[1]-$info[2];
                        	}
                        	else
                        	{
                        	       $actual_position=-($info[1]-$info[3]+$scalar);
	
                        	}
			}
			else
			{
				
                        	if( $info[2] < $info[3])
                        	{
                        	       $actual_position=-($info[1]-$info[2]+$scalar);
				 
                        	}
                        	else
                        	{
                        	       $actual_position=$info[1]-$info[3];
                        	}

			}
		}
		else
		{	
			if( $info[2] < $info[3])
                        {
                                 $actual_position=$info[1]-$info[2];
                        }
                        else
                       	{
                                 $actual_position=-($info[1]-$info[3]+$scalar);
                        }
		}
			
                print FINAL"$seq_id".","."$actual_position"."\n";
                print FINAL"$seq\n";
            }
        }
        else 
         {
                print FINAL "$_\n";
         }
}



close(FINAL);
close(OUT);
close(TEMP1);
untie @lines;

#removing temp directory
unlink("tmp/temp1") or die "Cannot delete temp file\n";
unlink ("tmp/outfile_temp") or die "Cannot delete temp file\n";
rmdir "tmp";


