#! /usr/bin/perl -w
use strict;
use IO::Handle;
use Getopt::Long;
use FileHandle;
use English;
use File::Basename;

########################################################################################################################
#
#Author : Vrunda Sheth
#
#
#Purpose : To make a parser which will parse the afg file and get the read tracking information for the adaptor
#
#
#Usage : perl solid_velvet_preprocessor.pl --file fragment_run file1 [--file fragment_run file2 ] (till how many ever files)
#	or perl solid_velvet_preprocessor.pl --run mates --f3 F3tag file --r3 r3tag file.
#
#
###########################################################################################################################


#options
use vars qw($opt_help);
use vars qw($opt_file);
use vars qw($opt_f3);
use vars qw($opt_r3);
use vars qw($opt_run);
use vars qw($opt_debug);
use vars qw($opt_version);
use vars qw($PROG         );



my @filenames;
my $option = "f";
my $version=1;
#$option=$opt_run if ($opt_run);
&GetOptions( "help|h",
             "file|f=s" => \@filenames,
             "f3=s",
	     "r3=s",
	     "run|r=s",
             "debug",
             "version|v",
           ) or die "Unknown option\n";

$PROG = basename($0);
$option=$opt_run if ($opt_run);
$version=$opt_version if ($opt_version);
#my $option=0;
if($opt_help)
{
print "\nUsage:$PROG [-help] --file csfasta file1 [--file csfasta file2].\n";
print "help        : print usage.\n";
print "run         : Input fragment/f for a fragment run and mates/m for a mate pair run\n";
print "file        : The csfasta file for your fragment run.\n";
print "f3          : The F3 tag file for a mate pair run.\n";
die   "r3          : The R3 tag  file for a mate pair run.\n";
}
if($opt_version)
{
	die "version is $version.0.0\n";
}
## Sanity check of input

if (! @filenames && ! $opt_f3 && ! $opt_r3 && ! $opt_version)
{
	print "\nUsage:$PROG [-help] --file csfasta file1 [--file csfasta file2].\n";
	die "Try $PROG -help for more information.\n\n";
}

if ($option eq 'F' || $option eq 'f' || $option eq 'Fragment' || $option eq 'fragment')
{
	if ( ! @filenames)
	{
     		die "ERROR1 : You have set the option for a fragment run. Please input csfasta file[s] with the --file switch.Try $PROG -help \n";
	}
	if ( $opt_f3 || $opt_r3)
	{
        	die "ERROR2 : You have set the option for a fragment run. Please input csfasta file[s] with the --file switch.Try $PROG -help \n";
	}
}

if ($option eq  'M' || $option eq 'm'|| $option eq 'Mates' || $option eq 'mates')
{
	if (  @filenames)
	{	
        	die "ERROR3 : You have set the option for a mate pair run. Please input f3 and r3 csfasta file with the --f3 and --r3 switches respectively.Try $PROG -help \n";
	}

	if ( $opt_f3 && ! $opt_r3 )
	{
		die "ERROR4 : Its a mate pair run. You need to submit R3 tag read file also. If you want to do a fragment run submit file[s] with the --file switch.Try $PROG -help \n";
	}
	if ($opt_r3 && ! $opt_f3)
	{
        	die "ERROR5 : Its a mate pair run. You need to submit F3 tag read file also. If you want to do a fragment run submit file[s] with the --file switch.Try $PROG -help \n";
	}
}

#globals

my $f3_file;
my $r3_file;
my $header;
my $seq;
my %myhash_f3;
my %myhash_r3;
my @verify;
$f3_file=$opt_f3;
$r3_file=$opt_r3;

# output directory for your csfasta and double encoded reads

mkdir("output", 0777) || print $!;

# colorspace_input.csfasta is the csfasta file that will go as input to teh post processor
#doubleEncoded_input.de is the double encoded file which will go as input to velvet.

open(CS,">output/colorspace_input.csfasta") or die "Cannot create file output/colorspace_input.csfasta\n";
open(DE,">output/doubleEncoded_input.de") or die "Cannot create file output/doubleEncoded_input.de\n";
if ($option eq 'F' || $option eq 'f'||$option eq 'Fragment' || $option eq 'fragment')
{

	foreach my $file (@filenames)
	{
		open (TEMP3,"$file") or die "Cannot open the input file submitted with --frag option\n";
		while(<TEMP3>)
		{
			chomp;
			if ( $_ !~ /^#/ )
			{
				print CS "$_\n";
			}
		}
	}

	close(CS);
	open(CS,"output/colorspace_input.csfasta") or die "Cannot read file output/colorspace_input.csfasta \n";
	while(<CS>)
	{
		chomp;
		if ( $_ =~ /^>/ )
		{
			print DE "$_\n";
		}
		else
		{
			$_=~s/\D\d//;
			$_=~tr/0123/ACGT/;
			print DE "$_\n";
		}
	}
}
else
{
	open (F3,"$f3_file") or die "Cannot open F3 tag file $!\n";
	 open (R3,"$r3_file") or die "Cannot open R3 tag file $!\n";
# Sanity check to make sure F3 and r3 files are supplied with the correct switches 
	while(<F3>)
	{
		chomp;
		if ( $_ =~ /^>/ )
		{
			 @verify=split(/_/,$_);
			if ($verify[$#verify] ne "F3")
			{
				die "ERROR6: You have not submitted the F3 tag file using the --f3 switch.\n";
			}
			else
			{
				last;
			}
		}
	}
	close(F3);
	 while(<R3>)
        {
                chomp;
                if ( $_ =~ /^>/ )
                {
                
                       @verify=split(/_/,$_);
                       if ($verify[$#verify] ne "R3")
                       {
                                die "ERROR7: You have not submitted the R3 tag file using the --r3 switch.\n";
		       }
		       else
		       {
				last;
		       }
		}
	}
	close(R3);  
	open (F3,"$f3_file") or die "Cannot open F3 tag file $!\n";
        open (R3,"$r3_file") or die "Cannot open R3 tag file $!\n";    
	
	while (<F3>)
	{
		chomp;
		if ( $_ !~ /^#/ )
		{
			if ( $_ =~ /^>/ )
			{			
				$header=$_;
				$header=~s/_F3//;
                	}
                	else
                	{
                        	   $seq=$_;
			}
	
			$myhash_f3{"$header"}=$seq;
			
	 	}
	}
        while(<R3>)
        {
                chomp;
		if ( $_ !~ /^#/ )
		{        
			if ( $_ =~ /^>/ )
                        {
                                $header=$_;
				$header=~s/_R3//;
			}
                        else
                        {
                                $seq=$_;
                        }
                        $myhash_r3{"$header"}=$seq;

                 }
        }
	
	foreach my $keys ( keys %myhash_r3)
	{
		if (exists $myhash_f3{$keys})
		{
			print CS "$keys"."_"."F3\n";
			print CS "$myhash_f3{$keys}\n";
			print CS "$keys"."_"."R3\n";
                        print CS "$myhash_r3{$keys}\n";
		}
	}
	close(CS);
	open(CS,"output/colorspace_input.csfasta") or die "Cannot read file output/colorspace_input.csfasta \n";
	my $count=0;
        while(<CS>)
        {
                chomp;
                if ( $_ =~ /^>/ )
                {
                        print DE "$_\n";
                }
                else
                {
                        $_=~s/\D\d//;
			if ($count %2 ==0)
			{
				$_=reverse($_);
			}	
                        $_=~tr/0123/ACGT/;
                        print DE "$_\n";
			$count++;
			
                }
        }
}
close(CS);
close(DE);

			
	
