#!/usr/bin/perl 

#AUTHOR
#  Adel Dayarian

#The code is available freely, under the GNU Public License, at http://www.physics.rutgers.edu/~anirvans/SOPRA/

# This code is the contig assembly part of S-SOPRA. It is based upon SSAKE.

use Storable;
use strict;
use Data::Dumper;

require "getopts.pl";
use vars qw($opt_m $opt_o $opt_v $opt_r $opt_z $opt_s $opt_t $opt_c $opt_h $opt_a $opt_e);
&Getopts('m:o:v:r:z:s:t:c:h:a:e:');
my ($base_overlap,$min_overlap,$verbose,$MIN_READ_LENGTH,$SEQ_SLIDE,$min_base_ratio,$seed_file,$max_trim,$base_name,$copy_li,$contig_size_cutoff,$ign_empi)=(3,16,0,16,1,0.7,"no-s",5,"",5,0,0);
my $per;
my $MAX = 0; 
my $MAX_TOP = 100; # this is the very maximum anchoring edge sequence that will be searched
#-------------------------------------------------    
if( ($opt_h) || (! $opt_a) ){
   print "Usage: $0\n";
   print "-m  Minimum number of overlapping bases with the current contig during overhang consensus build up for extension (default -m 16)\n";
   print "-a  The directory that you used for output of the formating script (format_base.pl)\n\n";   

   print "-o  Minimum number of reads needed to call a base during an extension (default -o 3)\n";
   print "-c  If the number of times a read and its reverse complement appear in the library is equal to or more than this value, the pairing information from that read will be disregarded  (default -c 5)\n";
   print "-r  Minimum base ratio used to accept a overhang consensus base (default -r 0.7)\n";
   print "-z  Minimum contig size (default 2*read_length, optional)\n";   
   print "-e  If set equal to 1, the empirical value for the insert size will not be used\n";   
   print "-t  Trim up to -t base(s) on the contig end when all possibilities have been exhausted for an extension (default -t 5)\n\n";

   #print "-s  Fasta file containing sequences to use as seeds exclusively (specify only if different from read set, optional)\n";
   exit;
}
my $date = `date`; chomp($date); print "Start $date\n"; 
$min_overlap = $opt_m if ($opt_m);
$base_overlap = $opt_o if ($opt_o);
$min_base_ratio = $opt_r if ($opt_r);
$max_trim = $opt_t if ($opt_t);
$verbose = $opt_v if ($opt_v);
$contig_size_cutoff = $opt_z if ($opt_z);
$seed_file = $opt_s if($opt_s);
$copy_li=$opt_c if($opt_c);
if( $opt_e == 1 ){
      $ign_empi=1;
}

my $address= $opt_a; 

my $display_seed_file = $1 if ($seed_file=~/([^\/]*)$/);
#-------------------------------------------------      
if( $address =~ /^(.*)\/$/ ){
     $address=$1; 
}
if (-d $address) {
	#print "There is a directory named $address\n";
}else {
	print "There is no such directory $address\n"; exit;
}

my $file = "$address"."/sread_in.fasta";
if(! -e $file){
   die "Can't open $file -- fatal\n";      
}
if($opt_s && ! -e $opt_s){
   die "The file $opt_s you specified does not exist -- fatal\n";
}

if($min_overlap < 11 || $min_overlap > 50){
   my $outofbound_message = "-m must be a number between 11-50 ...Exiting.\n";
   print $outofbound_message;
   exit;
}
if($base_overlap < 1){
   my $outofbound_message = "-o must be set to 1 or higher ...Exiting.\n";
   print $outofbound_message;
   exit;
}
if($min_base_ratio <= 0.5 || $min_base_ratio > 1){
   my $outofbound_message = "-r must be a number between 0.51 and 1.00 ...Exiting.\n";
   print $outofbound_message;
   exit;
}
#-------------------------------------------------
open (IN, "<$file");
my $du_line=<IN>;
my $paired=0;
if($du_line =~ /^Mate.(.*)_(\d*)$/){
   $paired=1; #print "\npaired $paired\n";
}
close IN;
#-------------------------------------------------
### Naming output files
my $par_name = "_m" . $min_overlap . "_o" . $base_overlap . "_r" . $min_base_ratio . "_t" . $max_trim . "_e" . $ign_empi ."_c$copy_li";
# if($paired){
#   $base_name .= "_d" . $insert_size . "_z" . $contig_size_cutoff . "_g-" . $display_unpaired_file;
# }
# if($opt_s){
#   $base_name .= "_s-" . $display_seed_file;
# }
my $pid_num = getpgrp(0);
$base_name .= "_pid" . $pid_num;

my $contig;

if ($paired){
   $contig="$address"."/contigs_raw". $base_name .".fasta" 
}else{
   $contig="$address"."/contigs". $base_name .".fasta" 
}

my $singlet = "$address"."/singlets". $base_name . $par_name .".fasta";
my $short_contig = "$address"."/shortcontigs".$base_name . ".fasta";

open (TIG, ">$contig") || die "Can't write to $contig -- fatal\n";
open (SIN, ">$singlet") || die "Can't write to $singlet -- fatal\n";
open (SHCON, ">$short_contig") || die "Can't write to $short_contig -- fatal\n";

mkdir("$address"."/div", 0777) || print $!;

open (CN, ">$address"."/div/copynumber.txt" ) or die "$!";
#-------------------------------------------------
my $init_message = "\nRunning: $0\n-f $file\n-m $min_overlap\n-c $copy_li\n-o $base_overlap\n-r $min_base_ratio\n-t $max_trim\n-e $ign_empi\n-a $address\n\n";
print $init_message;
print "pid $pid_num\n\n";  
#-------------------------------------------------
$date = `date`;
chomp($date);
my $reading_reads_message = "\nReading sequences initiated $date    pid $pid_num\n";
print $reading_reads_message;

my($set,$bin,$seed,$seq_l_u,$seq_l_d);
($set,$bin,$seq_l_u,$seq_l_d) = &readFasta($set,$bin,$file); 
#-------------------------------------------------
if($contig_size_cutoff==0){
	$contig_size_cutoff=int(2*$seq_l_d); 
}
print "max read length $seq_l_u bp      min read length $seq_l_d bp      contig_size_cutoff $contig_size_cutoff\n";
my $seq_l=$seq_l_u;
#-------------------------------------------------
### Allow user to specify a fasta file containing sequences to use as seeds, exclusively
my $real_seed=0;
if(-e $opt_s){
   my $use_seed_message = "Using seed sequence file $opt_s for this assembly.\nNote: ONLY sequences in $opt_s will be used as seeds\n";
   print $use_seed_message if ($verbose);
   $real_seed=1; 
   $seed = &loadSeed($opt_s); 
}else{
   $seed = $set;
}
my $seed_number_message = "Number of unique seed sequences: " . keys( %$seed ) . "\n";
printf $seed_number_message;
#-------------------------------------------------
$date = `date`;
chomp($date);

my $ssake_start_message = "\nSequence assembly initiated $date\n";
print $ssake_start_message;
#-------------------------------------------------
my ($R_sgl_count, $sgl_count,$tig_count,$previous_index) = (1,1,1,0); 
my $kharab1; my $kharab2; my $kharab3; my $kharab4; my $kharab5; 
my ($track_all, $tig_info);

eval{
my $status_bar = "+";
for(my $i=1;$i<=99;$i++){
   $per->{$i}++;
   my $ct = $i /10;
   if($ct == int($ct)){$status_bar .= $ct;}else{$status_bar .= "-";}
}
$status_bar .= "+ x 10 [% complete]";
print "$status_bar\n.";

my $keys_start = keys ( %$seed );
#---------------------------------------------  -AD
ASSEMBLY:
foreach my $seq (sort {$seed->{$b}{'count'}<=>$seed->{$a}{'count'}} keys %$seed){#cycle through the input [normalized] reads
   my $track;
   if(defined $seed->{$seq}){#sequence read hasn't been used, is longer than 11 nt and the user-defined overlap minimum -m
      my $seed_name = "";
      if(defined $seed->{$seq}{'name'}){$seed_name = "|seed:" . $seed->{$seq}{'name'};}
      my $orig_mer = length($seq);

	 #--------------------------------------------- 
	  my $seqrc = reverseComplement($seq); my @c_n;
	 
	  my $tcount=$set->{$seq}{'count'}; 
	  if(defined $set->{$seqrc}){
			$tcount+=$set->{$seqrc}{'count'};
	  }            
	  push @c_n, $tcount;  
	  
      if( $tcount < $copy_li){
          $track->{$seq}{'start'} = 1; $track->{$seq}{'end'} = $orig_mer; $track->{$seq}{'tig'} = $tig_count;
      }
      #### Delete keys ref
      my $start_sequence = $seq;
      my $reads_needed = $set->{$seq}{'count'};                      #tracks coverage
      my $total_bases = $orig_mer * $reads_needed;
      my $alone=1;
      ($bin,$set,$seed)=deleteData($bin,$set,$seed,$seq);   #remove k-mer from hash table and prefix tree
      
      if(defined $set->{$seqrc}){
			 $reads_needed +=$set->{$seqrc}{'count'};
			 $total_bases += length($seqrc) * $set->{$seqrc}{'count'};
			 ($bin,$set,$seed) = deleteData($bin,$set,$seed,$seqrc);
			 if(! defined $track->{$seqrc}){ 
				if($tcount<$copy_li){
					   $track->{$seqrc}{'tig'} = $tig_count;
					   $track->{$seqrc}{'end'} = 1;
					   $track->{$seqrc}{'start'} = $orig_mer;
				}
			 }      
      }
    #---------------------------------------------           
      print "\n\n>>> START SEED SEQUENCE :: $seq <<<\n\n" if ($verbose);
      
      ($seq, $set, $bin, $reads_needed, $total_bases, $track, $alone, @c_n) = doExtension("3", $orig_mer, $seq, $set, $bin, $reads_needed, $total_bases, $min_overlap, $base_overlap, $min_base_ratio, $verbose, $track, $tig_count, $max_trim, $alone, @c_n);
      ####end of 3' extension, beginning of 5' extension  (via 3' RC)
      
      $seqrc = reverseComplement($seq);
      ($seqrc, $set, $bin, $reads_needed, $total_bases, $track, $alone, @c_n) = doExtension("5", $orig_mer, $seqrc, $set, $bin, $reads_needed, $total_bases, $min_overlap, $base_overlap, $min_base_ratio, $verbose, $track, $tig_count, $max_trim, $alone, @c_n);
      ####end of 5' extension

      my $leng = length($seqrc);
      my $reversetig = reverseComplement($seqrc);                   ### return to sequence, as inputted
      #--------------------------------	             
      if( $leng >= $contig_size_cutoff ){ 
         my $cov =  int($total_bases / $leng);
         printf TIG ">contig%i|size%i|read%i|cov%.2f|$seed_name\n%s\n", ($tig_count,$leng,$reads_needed,$cov,$reversetig);    #print contigs to file

         $tig_info->{$tig_count}{'cov'}=$cov;
         $tig_info->{$tig_count}{'length'}= $leng;
         if($paired){
               $track_all = &trackReads($track,$track_all);
         }
         $tig_count++;

		  print CN "@c_n ";        
         
      }else{
         if ($alone){
            my $leng = length($start_sequence);
            printf SIN ">singlet%i|size%i|read%i\n%s\n", ($R_sgl_count,$leng,$reads_needed,$start_sequence); 
            $R_sgl_count++;
         }else{   	 
             my $cov =  $total_bases / $leng;
			 printf SHCON ">short%i|size%i|read%i|cov%.2f|$seed_name\n%s\n", ($sgl_count,$leng,$reads_needed,$cov,$reversetig);    #print singlets to file
			 $sgl_count++;
         }         
      }
   } 
   
   my $keys_left = keys( %$seed );
   my $index = (int((($keys_start-$keys_left)/$keys_start)*100));
   if(defined $per->{$index}){
      print "." x ($index - $previous_index);
      $|=1; ###clear buffer
      delete $per->{$index};
   }
   $previous_index = $index;
   last ASSEMBLY if (! $keys_left);
}
print ".";
};###end eval block

$date = `date`; chomp($date);

if($@){
   my $message = $@;
   my $failure = "\nSomething went wrong running $0 $date\n$message\n";
   print $failure;
}else{
   my $success = "\nContig assembly executed normally $date\n";
   print $success;
}
close TIG; close SIN; close CN;

if(! $paired){   
    store \%{$tig_info}, "$address"."/tig_info_$base_name"; 
    
	my @lengths; my $sum;
	foreach my $a (keys %$tig_info){
		 push @lengths, $tig_info->{$a}{'length'} ;
		 $sum+=$tig_info->{$a}{'length'} ;
	}	
	@lengths=sort {$b <=> $a} @lengths;  
	
	my $larges=0;my $N50;
	
	SBASSE: for (my $i=0; $i < scalar(@lengths); $i++){        
		  $larges+=$lengths[$i];
		  if($larges>($sum/2)){
				$N50=$lengths[$i];
				last SBASSE;
		  }
	}    
   	my $date = `date`; chomp($date);  print "\nFinished $date     N50 $N50     sum $sum\n\n";    
	
    exit;
}elsif($real_seed==0){     
    my $date = `date`; chomp($date);   print "Self-consistency check for contigs $date\n";
    
    my $N_track_all; my $N_tigcount=0; my $forbiden_link; my $N_tig_info; my $kibeki; my $num_m_file; my $unique_insert_size;  
    
    ($tig_info,$num_m_file,$unique_insert_size)=ShadowPatern($file, $track_all, $tig_count, $tig_info); 
    #print "shadow pattern done num_m_file $num_m_file unique_insert_size $unique_insert_size\n"; 
    undef $tig_count; 
    store \%{$tig_info}, "$address"."/div/tig_info_raw".$base_name; 

	my $mean=0; my $dus=0; my $N=0; my $cutoff_selfin;
	foreach my $tig (keys %{$tig_info}){  
		$mean+=$tig_info->{$tig}{'in_consis'}; $dus+=$tig_info->{$tig}{'in_consis'}**2; $N++;  
	}
	$mean=int($mean/$N);  $dus=int($dus/$N); my $std=int(($dus-$mean**2)**.5);  
	#print "self mean $mean std $std N $N\n";
	
	if((2*$mean+3*$std)>21){
	   $cutoff_selfin=2*$mean+3*$std;  #print "cutoff_selfin $cutoff_selfin = 2*$mean+3*$std\n";
	}else{
	   $cutoff_selfin=21;  
	} 
   
    my $N_contig = "$address"."/contigs". $base_name .".fasta"; 
    
    open (TIG, "<$contig") or die "$!"; open (NTIG, ">$N_contig") || die "$!";
    while (<TIG>){  
	   my $tig_o; my $old_contig; 
	   if ($_=~ /^\>/){
			my @string = split(/\|/,$_);
			$tig_o=substr($string[0],7);
			$old_contig=<TIG>; chomp($old_contig);
	   } 
	   if(($tig_o % 200)==0){
	        chomp($date); print "contig $tig_o\n";  
	   }
	  
		   my $problem=1;
		   if(defined $tig_info->{$tig_o}{'tot_w'}){	
			  	
				my ($left_k, $right_k, $tot_Tnew, $m_starw, $m_endw, $starw, $endw); 
				 
				($left_k,$right_k,$tot_Tnew,$tig_info, $m_starw, $m_endw, $starw, $endw)=Analyzecontig($tig_o,$tig_info,$unique_insert_size);	
				
				if ($tot_Tnew==1){
					if(($left_k>0) || ($right_k>0)){
					    if($tig_info->{$tig_o}{'in_consis'}>$cutoff_selfin){
					           if(($tig_info->{$tig_o}{'length'}-$seq_l*($left_k+$right_k)) <= (3*$unique_insert_size)){
					                 $problem=0;  
					           } 
					    }
					    if($problem==1){

							my $N_string; 
							$N_string->{'start'}=$left_k*$seq_l+1;
							$N_string->{'end'}=$tig_info->{$tig_o}{'length'}-$right_k*$seq_l; 
							if(($N_string->{'end'} - $N_string->{'start'})>0 ){   
								$N_string->{'string'}=substr($old_contig, $N_string->{'start'}-1, $N_string->{'end'} - $N_string->{'start'}+1); 
								#print "\ntot_new $tot_Tnew - 1 : s $N_string->{1}{'start'} e $N_string->{1}{'end'} ori end\n\n";
								my $leng=$N_string->{'end'} - $N_string->{'start'}+1; 
								if( $leng>$contig_size_cutoff ){
									$N_tigcount++;
									$N_tig_info->{$N_tigcount}{'length'}=$leng;
									$N_tig_info->{$N_tigcount}{'cov'}=$tig_info->{$tig_o}{'cov'};				
									printf NTIG ">contig%i|size%i|cov%.2f\n%s\n", ($N_tigcount,$N_tig_info->{$N_tigcount}{'length'},$N_tig_info->{$N_tigcount}{'cov'},$N_string->{'string'});    #print contigs to file
									 
									foreach my $read (keys %{$track_all}){
										if($tig_o==$track_all->{$read}{'tig'}){
											 my $A_start = $track_all->{$read}{'start'};
											 my $A_end = $track_all->{$read}{'end'};
											 if($A_start>=$N_string->{'start'} && $A_end>=$N_string->{'start'} && $A_start<=$N_string->{'end'} && $A_end<=$N_string->{'end'}){
												 $N_track_all->{$read}{'tig'}=$N_tigcount; 
												 $N_track_all->{$read}{'start'}=$track_all->{$read}{'start'}- $N_string->{'start'}+1;
												 $N_track_all->{$read}{'end'}=$track_all->{$read}{'end'}- $N_string->{'start'}+1; 
											 }
											 delete $track_all->{$read};
										}
									}
									foreach my $read (keys %{$track_all}){
										if (!defined $track_all->{$read}{'tig'}){    
										   delete $track_all->{$read};
										}
									}
									delete $tig_info->{$tig_o};
								}else{
									 printf SHCON ">short%i|size%i|cov%.2f\n%s\n", ($sgl_count,$leng,$tig_info->{$tig_o}{'cov'},$N_string->{'string'});    #print singlets to file
									 $sgl_count++; 
									 delete $tig_info->{$tig_o}; 
									 foreach my $read (keys %{$track_all}){
											if($tig_o==$track_all->{$read}{'tig'}){
												 delete $track_all->{$read};
											}
									 }
								}	
							}else{
								 #print "old_t $tig_o negative l_k $left_k r_k $right_k star $N_string->{'start'} end $N_string->{'end'} tot $tot_Tnew m_s $m_starw m_e $m_endw s $starw e $endw\n";
							}
						}	
					}else{
					    $problem=0; 
					}
				}else{
                    if( ($tig_info->{$tig_o}{'length'}-($left_k+$right_k)*$seq_l) > 0){
						my @list_rep; my $N_string; 
						my $start_newc=$left_k;  
						my $end_newc=shift(@{$tig_info->{$tig_o}{'hump'}{1}{'mloc'}})-1;  
						$N_string->{1}{'start'}=$start_newc*$seq_l+1;
						$N_string->{1}{'end'}=$end_newc*$seq_l;   
						if((($N_string->{1}{'end'} - $N_string->{1}{'start'})>0) && ($N_string->{1}{'start'}>0) && ($N_string->{1}{'end'}<=$tig_info->{$tig_o}{'length'})){   
							$N_string->{1}{'string'}=substr($old_contig, $N_string->{1}{'start'}-1, $N_string->{1}{'end'} - $N_string->{1}{'start'}+1); 
							my $leng=$N_string->{1}{'end'} - $N_string->{1}{'start'}+1;
							if( $leng>$contig_size_cutoff){
								$N_tigcount++;
								$N_string->{1}{'new'}=$N_tigcount;
								$N_tig_info->{$N_tigcount}{'length'}=$leng;
								$N_tig_info->{$N_tigcount}{'cov'}=$tig_info->{$tig_o}{'cov'};
								push @list_rep, $N_tigcount;
								printf NTIG ">contig%i|size%i|cov%.2f\n%s\n", ($N_tigcount,$N_tig_info->{$N_tigcount}{'length'},$N_tig_info->{$N_tigcount}{'cov'},$N_string->{1}{'string'});    #print contigs to file
							}else{
								 printf SHCON ">short%i|size%i|cov%.2f\n%s\n", ($sgl_count,$leng,$tig_info->{$tig_o}{'cov'},$N_string->{1}{'string'});    #print singlets to file
								 $sgl_count++;  
								 delete $N_string->{1};
							}
						}else{
						
						}	
						$start_newc=pop(@{$tig_info->{$tig_o}{'hump'}{$tot_Tnew-1}{'mloc'}}); 
						$N_string->{$tot_Tnew}{'start'}=$start_newc*$seq_l;
						$N_string->{$tot_Tnew}{'end'}=length($old_contig)-$right_k*$seq_l;  
						if((($N_string->{$tot_Tnew}{'end'} - $N_string->{$tot_Tnew}{'start'})>0) && ($N_string->{$tot_Tnew}{'start'}>0) && ($N_string->{$tot_Tnew}{'end'}<=$tig_info->{$tig_o}{'length'})){   
							$N_string->{$tot_Tnew}{'string'}=substr($old_contig, $N_string->{$tot_Tnew}{'start'}-1, $N_string->{$tot_Tnew}{'end'} - $N_string->{$tot_Tnew}{'start'}+1); 
							my $leng=$N_string->{$tot_Tnew}{'end'} - $N_string->{$tot_Tnew}{'start'}+1; 
							if( $leng>$contig_size_cutoff){
								$N_tigcount++;
								$N_string->{$tot_Tnew}{'new'}=$N_tigcount;
								$N_tig_info->{$N_tigcount}{'length'}=$leng;
								$N_tig_info->{$N_tigcount}{'cov'}=$tig_info->{$tig_o}{'cov'};
								push @list_rep, $N_tigcount;
								printf NTIG ">contig%i|size%i|cov%.2f\n%s\n", ($N_tigcount,$N_tig_info->{$N_tigcount}{'length'},$N_tig_info->{$N_tigcount}{'cov'},$N_string->{$tot_Tnew}{'string'});    #print contigs to file
							}else{
								 printf SHCON ">short%i|size%i|cov%.2f\n%s\n", ($sgl_count,$leng,$tig_info->{$tig_o}{'cov'},$N_string->{$tot_Tnew}{'string'});    #print singlets to file
								 $sgl_count++; 
								 delete $N_string->{$tot_Tnew};
							} 
						}else{
						   
						}
						for(my $T_newc=2; $T_newc < $tot_Tnew; $T_newc++){
							  my $start_newc=pop(@{$tig_info->{$tig_o}{'hump'}{$T_newc-1}{'mloc'}});  
							  my $end_newc=shift(@{$tig_info->{$tig_o}{'hump'}{$T_newc}{'mloc'}})-1;  
							  $N_string->{$T_newc}{'start'}=$start_newc*$seq_l;
							  $N_string->{$T_newc}{'end'}=$end_newc*$seq_l; 
							  if((($N_string->{$T_newc}{'end'} - $N_string->{$T_newc}{'start'})>0) && ($N_string->{$T_newc}{'start'}>0) && ($N_string->{$T_newc}{'end'}<=$tig_info->{$tig_o}{'length'}) ){   
								  $N_string->{$T_newc}{'string'}=substr($old_contig, $N_string->{$T_newc}{'start'}-1, $N_string->{$T_newc}{'end'} - $N_string->{$T_newc}{'start'}+1); 
								  my $leng=$N_string->{$T_newc}{'end'} - $N_string->{$T_newc}{'start'}+1; 
								  if( $leng>$contig_size_cutoff){
									  $N_tigcount++;
									  $N_string->{$T_newc}{'new'}=$N_tigcount;
									  $N_tig_info->{$N_tigcount}{'length'}=$leng;
									  $N_tig_info->{$N_tigcount}{'cov'}=$tig_info->{$tig_o}{'cov'};
									  push @list_rep, $N_tigcount;		
									  printf NTIG ">contig%i|size%i|cov%.2f\n%s\n", ($N_tigcount,$N_tig_info->{$N_tigcount}{'length'},$N_tig_info->{$N_tigcount}{'cov'},$N_string->{$T_newc}{'string'});    #print contigs to file								 
								  }else{
									  printf SHCON ">short%i|size%i|cov%.2f\n%s\n", ($sgl_count,$leng,$tig_info->{$tig_o}{'cov'},$N_string->{$T_newc}{'string'});    #print singlets to file
									  $sgl_count++; 
									  delete $N_string->{$T_newc};
								  }
							  }else{
						
							  }
						}
				
						delete $tig_info->{$tig_o};
						foreach my $read (keys %{$track_all}){
							if($tig_o==$track_all->{$read}{'tig'}){
								 my $A_start = $track_all->{$read}{'start'};
								 my $A_end = $track_all->{$read}{'end'};
								 foreach my $newc (keys %{$N_string}){
								    if(defined $N_string->{$newc}{'start'}){
										 if($A_start>=$N_string->{$newc}{'start'} && $A_end>=$N_string->{$newc}{'start'} && $A_start<=$N_string->{$newc}{'end'} && $A_end<=$N_string->{$newc}{'end'}){
											 $N_track_all->{$read}{'tig'}=$N_string->{$newc}{'new'}; 
											 $N_track_all->{$read}{'start'}=$track_all->{$read}{'start'}- $N_string->{$newc}{'start'}+1; ; 
											 $N_track_all->{$read}{'end'}=$track_all->{$read}{'end'}- $N_string->{$newc}{'start'}+1; ; 
										 }
								    }		 
								 }	 
								 delete $track_all->{$read};
							}
						}
						foreach my $read (keys %{$track_all}){
							if (!defined $track_all->{$read}{'tig'}){    
							   delete $track_all->{$read};
							}
						}						
						foreach my $m (@list_rep){
							foreach my $n (@list_rep){
								$forbiden_link->{$m}{$n}=1; $forbiden_link->{$n}{$m}=1;
							}
						}	
					}else{
					     
					}
				}
		   }else{
		        $problem=0;
		   }
		   if($problem==0){	   
		   	   if($tig_info->{$tig_o}{'in_consis'}>(3*$cutoff_selfin)){
						  #print "selfinconsis $tig_o $tig_info->{$tig_o}{'in_consis'} L $tig_info->{$tig_o}{'length'} dup 0 problem $problem\n";	  
			   }elsif(($tig_info->{$tig_o}{'in_consis'}>$cutoff_selfin) && ($tig_info->{$tig_o}{'length'}<3000)){

			  } 	
			   $N_tigcount++;  $kibeki->{$tig_o}=$N_tigcount;
			   $N_tig_info->{$N_tigcount}{'length'}=$tig_info->{$tig_o}{'length'};
			   $N_tig_info->{$N_tigcount}{'cov'}=$tig_info->{$tig_o}{'cov'};
			   delete $tig_info->{$tig_o};
			   printf NTIG ">contig%i|size%i|cov%.2f\n%s\n", ($N_tigcount,$N_tig_info->{$N_tigcount}{'length'},$N_tig_info->{$N_tigcount}{'cov'},$old_contig);    #print contigs to file
		   }
    }
    
    foreach my $read (keys %{$track_all}){
        my $ghadimi=$track_all->{$read}{'tig'};
	    if (defined $kibeki->{$ghadimi}){    
		   $N_track_all->{$read}{'tig'}=$kibeki->{$ghadimi}; 
		   $N_track_all->{$read}{'start'}=$track_all->{$read}{'start'}; 
		   $N_track_all->{$read}{'end'}=$track_all->{$read}{'end'}; 
		   delete $track_all->{$read};
	    }elsif($tig_info->{$ghadimi}{'in_consis'}>$cutoff_selfin){
	       
	    }else{
	       #print "ghadimi $ghadimi - $tig_info->{$ghadimi}{'in_consis'}\n";
	    }
    }
    close TIG; close NTIG; close SHCON;

	undef $track_all; undef $tig_info; my ($Ori_Dis, $selfpair); print "\n";

    my @aray_ins = Empiricalsize($file, $N_track_all, $forbiden_link, $N_tig_info, $N_tigcount);  

	($Ori_Dis, $selfpair, $N_tig_info)= OrientationDistance($file, $N_track_all, $forbiden_link, $N_tig_info, $N_tigcount, @aray_ins );  
	
	store \%{$Ori_Dis}, "$address"."/orientdistinfo". $base_name; 
	undef $Ori_Dis; 
	#print "selfpair $selfpair\n\n\n1 $kharab1 2 $kharab2 3 $kharab3 4 $kharab4 lil_copy $kharab5\n\n\n";
	
	open (SF, ">$address"."/badselfpair$base_name".".txt") or die "$!";       
	foreach my $a (keys %$N_tig_info){
	    if($N_tig_info->{$a}{'in_consis'}>$cutoff_selfin){
	       print SF "contig $a number of bad pair $N_tig_info->{$a}{'in_consis'}\n";
	    }
	}
	close SF;

    $N_tig_info->{1}{'seq_l_u'}=$seq_l_u;
    $N_tig_info->{1}{'seq_l_d'}=$seq_l_d;
	
	my @lengths; my $sum;
	foreach my $a (keys %$N_tig_info){
		 push @lengths, $N_tig_info->{$a}{'length'} ;
		 $sum+=$N_tig_info->{$a}{'length'} ;
	}	
	@lengths=sort {$b <=> $a} @lengths;  
	
	my $larges=0;my $N50;
	
	SBASSE: for (my $i=0; $i < scalar(@lengths); $i++){        
		  $larges+=$lengths[$i];
		  if($larges>($sum/2)){
				$N50=$lengths[$i];
				last SBASSE;
		  }
	}
    
	store \%{$N_tig_info}, "$address"."/div/tig_info". $base_name; 

	open (CDIS, ">$address"."/div/coverage_distribution.txt");
	
	foreach my $tig ( keys %{$N_tig_info}){  
		  print CDIS "$N_tig_info->{$tig}{'cov'} ";
	}

	close CDIS;

	$date = `date`; chomp($date); print "\nFinished $date     N50 $N50    sum $sum\n\n";    

	exit;
}else{

}
 
exit;
#---------------------------------------------------      
sub Analyzecontig{
    my ($tig_a, $tig_info,$insert_size) = @_;  			
	my $tot_Tnewc=1; my ($chap_k, $rast_k, $bikhial, $m_start_w, $m_end_w, $start_w, $end_w)=(0, 0, 0, 0, 0, 0, 0); 
	
	my $tmean=0;  
	for(my $i=1;$i<=$tig_info->{$tig_a}{'tot_w'};$i++){
		$tmean+=$tig_info->{$tig_a}{'pilar'}{$i};  
	}
	$tmean= int($tmean/$tig_info->{$tig_a}{'tot_w'});		
	
	if($tmean<50){
	    #print "\ntig_a $tig_a tot mean $tmean\n";
	}else{
		DLS: for(my $i=1;$i<=$tig_info->{$tig_a}{'tot_w'};$i++){
			if($tig_info->{$tig_a}{'pilar'}{$i}<1){
				 $chap_k++;
			}else{
				 last DLS;
			}
		}
		DRS: for(my $i=$tig_info->{$tig_a}{'tot_w'};$i>=1;$i--){
			if($tig_info->{$tig_a}{'pilar'}{$i}<1){
				 $rast_k++;
			}else{
				 last DRS;
			}
		}

		if(($tig_info->{$tig_a}{'length'}-$seq_l*($chap_k+$rast_k))>(3*$insert_size)){
			my $hol=1;
		   REQU: while($hol==1){
					$hol=0;
					$m_start_w=int($insert_size/$seq_l) +$chap_k; $m_end_w=$tig_info->{$tig_a}{'tot_w'}-int($insert_size/$seq_l)-$rast_k; 
					$start_w=$m_start_w; $end_w=$m_end_w;  
		
					my $mean_pil=0; my $dus=0;  
					for(my $i=$m_start_w; $i<=$m_end_w; $i++){
						$mean_pil+=$tig_info->{$tig_a}{'pilar'}{$i}; $dus+=$tig_info->{$tig_a}{'pilar'}{$i}**2;
					}
					$mean_pil= int($mean_pil/($m_end_w-$m_start_w+1)); $dus=int($dus/($end_w-$start_w+1));  my $std=int(($dus-$mean_pil**2)**.5); my $cut_pil; 
					if(($mean_pil-4*$std)>(.5*$mean_pil)){
						$cut_pil=$mean_pil-4*$std; 
					}else{
						$cut_pil=int(.5*$mean_pil);
					} 
					PUSHLE: for(my $i=$m_start_w; $i<=$m_end_w; $i++){
							if($tig_info->{$tig_a}{'pilar'}{$i}>$cut_pil){
								last PUSHLE;
							}else{
								$start_w++;
							}
					}
					PUSHRI: for(my $i=$m_end_w; $i>$start_w; $i--){
							if($tig_info->{$tig_a}{'pilar'}{$i}>$cut_pil){
								last PUSHRI;
							}else{
								$end_w--;
							}
					}	    	
					FMINL: for(my $i=$start_w-1; $i>$chap_k; $i--){
							 my $minima=1; 
							 for(my $j=1; $j<5; $j++){
								   if($tig_info->{$tig_a}{'pilar'}{$i-$j}<$tig_info->{$tig_a}{'pilar'}{$i}){
										$minima=0; 
								   }
							 }
							 if($minima==1){
								 #print "tig_o $tig_a moved the leftk from $chap_k to $i\n";
								 $chap_k=$i;  
								 $hol=1;
								 last FMINL;
							 }
					}  	
					FMINR: for(my $i=$end_w+1; $i< ($tig_info->{$tig_a}{'tot_w'}-$rast_k); $i++){
							 my $minima=1; 
							 for(my $j=1; $j<5; $j++){
								   if($tig_info->{$tig_a}{'pilar'}{$i+$j}<$tig_info->{$tig_a}{'pilar'}{$i}){
										$minima=0; 
								   }
							 }
							 if($minima==1){
								 my $du=$tig_info->{$tig_a}{'tot_w'}-$i+1;
								 #print "tig_o $tig_a moved the rightk from $rast_k to $du\n";
								 $rast_k=$tig_info->{$tig_a}{'tot_w'}-$i+1;
								 $hol=1;
								 last FMINR;
							 }
					}
					if(($tig_info->{$tig_a}{'length'}-$seq_l*($chap_k+$rast_k))<(3*$insert_size)){
						#print "tig_a $tig_a s type l $tig_info->{$tig_a}{'length'}  l $chap_k r $rast_k - s $start_w e $end_w\n"; 
						$bikhial=1; last REQU;
					}
					if( ($seq_l*($end_w-$start_w))<(3*$insert_size)){
						#print "tig_a $tig_a s type e-s  l $chap_k r $rast_k - s $start_w e $end_w\n"; 
						$bikhial=1; last REQU;
					}
					if($start_w>$end_w){
						#print "tig_a $tig_a s greater than e l $chap_k r $rast_k  - s $start_w e $end_w\n"; 
						$bikhial=1; last REQU;
					}
					if($start_w<0 || 0>$end_w){
						#print "tig_a $tig_a s or e smaler than zero l $chap_k r $rast_k - s $start_w e $end_w\n"; 
						$bikhial=1; last REQU;
					}
			}		
			if( ($bikhial==0) && (($tig_info->{$tig_a}{'length'}-$seq_l*($chap_k+$rast_k))>(3*$insert_size)) ){
				my $mean_pil=0; my $dus=0;  
				for(my $i=$start_w; $i<=$end_w; $i++){
					$mean_pil+=$tig_info->{$tig_a}{'pilar'}{$i}; $dus+=$tig_info->{$tig_a}{'pilar'}{$i}**2;
				}
				$mean_pil= int($mean_pil/($end_w-$start_w+1)); $dus=int($dus/($end_w-$start_w+1));  my $std=int(($dus-$mean_pil**2)**.5);   my $cut_pil;  
				if(($mean_pil-4*$std)>(.5*$mean_pil)){
					$cut_pil=$mean_pil-4*$std; 
				}else{
					$cut_pil=int(.5*$mean_pil);
				} 			
		
				if($mean_pil<50){
					#print "tig_a $tig_a mean_pil $mean_pil\n"; 
					$bikhial=1;
				}
				my $hump=0; my $level;
				for(my $i=$start_w; $i<=$end_w; $i++){
					if($tig_info->{$tig_a}{'pilar'}{$i}<$cut_pil){
						$level->{$i}=0;
					}else{
						$level->{$i}=1;
					}
				}
				if($level->{$end_w}==0 || $level->{$start_w}==0){
				    #print "\ntig_o $tig_a s $start_w level $level->{$start_w} e $end_w level  $level->{$end_w}\n";
					PUSHLE1: for(my $i=$start_w; $i<=$end_w; $i++){
							if($tig_info->{$tig_a}{'pilar'}{$i}>$cut_pil){
								last PUSHLE1; #print "\nleft is not a problem\n";
							}else{
								$start_w++;
							}
					}
					PUSHRI2: for(my $i=$end_w; $i>$start_w; $i--){
							if($tig_info->{$tig_a}{'pilar'}{$i}>$cut_pil){
								last PUSHRI2; #print "\nleft is not a problem\n";
							}else{
								$end_w--;
							}
					}
					#print "\s $start_w e $end_w\n";	
				}				
				if($level->{$end_w}==0 || $level->{$start_w}==0){
					#print "\ntig_o $tig_a s $start_w level $level->{$start_w} $tig_info->{$tig_a}{'pilar'}{$start_w} e $end_w level $level->{$end_w} $tig_info->{$tig_a}{'pilar'}{$end_w} -  cut_pil $cut_pil ms $m_start_w  me $m_end_w\n";

					$bikhial=1; #print "\ntig_o $tig_a bikhial $bikhial cause of level\n";
				}
				if(($bikhial==0) && ( ($seq_l*($end_w-$start_w)) > (3*$insert_size) ) ){
					for(my $i=$start_w+1; $i<=$end_w-1; $i++){
						 if($level->{$i}==0){
							  if($level->{$i-1}==1){
								  $hump++;
								  $tig_info->{$tig_a}{'hump'}{$hump}{'start'}=$i;
								  $tig_info->{$tig_a}{'hump'}{$hump}{'min'}=$tig_info->{$tig_a}{'pilar'}{$i};
							  }
							  if($tig_info->{$tig_a}{'pilar'}{$i}<$tig_info->{$tig_a}{'hump'}{$hump}{'min'}){
								  $tig_info->{$tig_a}{'hump'}{$hump}{'min'}=$tig_info->{$tig_a}{'pilar'}{$i};
							  }
							  if($level->{$i+1}==1){
								  $tig_info->{$tig_a}{'hump'}{$hump}{'end'}=$i;
								  if(($tig_info->{$tig_a}{'hump'}{$hump}{'end'}-$tig_info->{$tig_a}{'hump'}{$hump}{'start'}) < 6){
										  #print "tig_a $tig_a delete hump $hump s $tig_info->{$tig_a}{'hump'}{$hump}{'start'} e $tig_info->{$tig_a}{'hump'}{$hump}{'end'} c $cut_pil m $mean_pil std $std\n";
										  delete $tig_info->{$tig_a}{'hump'}{$hump};
										  $hump--;
								  }
							  }
						 }
					}
					$tot_Tnewc+=$hump; undef $hump;
					REDMIN: foreach my $h (keys %{$tig_info->{$tig_a}{'hump'}}){
						for(my $i=$tig_info->{$tig_a}{'hump'}{$h}{'start'}; $i< $tig_info->{$tig_a}{'hump'}{$h}{'end'}; $i++){
							 if($tig_info->{$tig_a}{'pilar'}{$i}==$tig_info->{$tig_a}{'hump'}{$h}{'min'}){
							   push @{$tig_info->{$tig_a}{'hump'}{$h}{'mloc'}}, $i; push @{$tig_info->{$tig_a}{'hump'}{$h}{'mloc'}}, $i;
							 }
						}
						if(defined @{$tig_info->{$tig_a}{'hump'}{$h}{'mloc'}}){
							@{$tig_info->{$tig_a}{'hump'}{$h}{'mloc'}} = sort {$a <=> $b} @{$tig_info->{$tig_a}{'hump'}{$h}{'mloc'}};
						}else{
							$bikhial=1; 
							#print "\ntig_o $tig_a bikhial $bikhial cause not defined c $cut_pil m $mean_pil std $std  start $start_w end $end_w\n";
							foreach my $h (keys %{$tig_info->{$tig_a}{'hump'}}){
								 #print "tig_a $tig_a hump $h $tig_info->{$tig_a}{'hump'}{$h}{'start'}  $tig_info->{$tig_a}{'hump'}{$h}{'end'}\n";
							}					
							last REDMIN;
						}
					}
					if($bikhial==1){
						  delete $tig_info->{$tig_a}{'hump'}; $tot_Tnewc=1; 
						  #print "\ntig_o $tig_a cause of bikhial $bikhial deleted the hump\n";
					}
				}
		# 		foreach my $h (keys %{$tig_info->{$tig_a}{'hump'}}){
		# 			print "\nhump $h start $tig_info->{$tig_a}{'hump'}{$h}{'start'} end $tig_info->{$tig_a}{'hump'}{$h}{'end'} min $tig_info->{$tig_a}{'hump'}{$h}{'min'} mloc @{$tig_info->{$tig_a}{'hump'}{$h}{'mloc'}}\n\n";
		# 		}
			}
		}	 
	}
	return $chap_k, $rast_k, $tot_Tnewc, $tig_info, $m_start_w, $m_end_w, $start_w, $end_w;		
}
#----------------------- 
sub  ShadowPatern{    
   my ($file, $track, $mtig, $tig_info) = @_;
   
   my $num_mate_file=0; my $insert_size; my $dev_2; my $dev_3;
   
   open(IN,$file) || die "Can't open $file -- fatal\n";
   
   while(my $line=<IN>){
		  chomp($line);
		  if($line =~ /^Mate.(.*)_(\d*)$/){
		       $num_mate_file++;
		       $insert_size=$2;
			   if($insert_size>1000){
				   $dev_3=3;
			   }else{
				   $dev_3=6;
			   }     
		  }
   }
   close IN;
   #print "num_mate_file $num_mate_file insert_size $insert_size\n";   
  
   if ($num_mate_file == 1){
	   my $mean=0; my $N=0; my $du=0; my $du1=0;
	   foreach my $tig (keys %{$tig_info}){  
			$mean+=$tig_info->{$tig}{'cov'};  $N++; $tig_info->{$tig}{'in_consis'}=0;
	   }
	   $mean=int($mean/$N); my $cut=2*$mean; #print "mtig $mtig mean_cov $mean cut $cut\n";
	   for(my $tig=1;$tig<$mtig;$tig++){
		   if(($tig_info->{$tig}{'length'}>($dev_3*$insert_size)) && ($tig_info->{$tig}{'cov'}<$cut) ){
			   my $tot_window=int($tig_info->{$tig}{'length'}/$seq_l);
			   $tig_info->{$tig}{'tot_w'}=$tot_window; 
			   for(my $i=1;$i<=$tig_info->{$tig}{'tot_w'};$i++){
				   $tig_info->{$tig}{'pilar'}{$i}=0; 
			   }
		   }
	   }
	
	   open(IN,$file) || die "Can't open $file -- fatal\n";
	   AKHARESH: while(my $line=<IN>){
		  chomp($line);
		  if($line =~ /^Mate.(.*)_(\d*)$/){
			  $insert_size=$2; 
			  if($insert_size>5000){
				   $dev_2=.2;
			  }elsif($insert_size>1000){
				   $dev_2=.5;
			  }else{
				   $dev_2=1;
			  }          
			 # print "insert_size $insert_size dev_2 $dev_2\n";		
		  }elsif($line =~ /^Fragment$/){
			  last AKHARESH;     
		  }elsif($line =~ /^([^\>]*)$/i){  
			 my $sdna = $1;
			 if($sdna =~/([ACGT]*)\:([ACGT]*)$/i){   
				my $read_a = $2; $du1++;
				my $read_b = $1;
				if(defined $track->{$read_a} && $track->{$read_b}){### both pairs assembled
					if(defined $track->{$read_a}{'tig'} && $track->{$read_b}{'tig'}){### both pairs assembled
						 my $tig_a = $track->{$read_a}{'tig'};
						 my $tig_b = $track->{$read_b}{'tig'};    
						 my $A_length = $tig_info->{$tig_a}{'length'};
						 my $A_start = $track->{$read_a}{'start'};
						 my $A_end = $track->{$read_a}{'end'};
						 my $B_length = $tig_info->{$tig_b}{'length'};
						 my $B_start = $track->{$read_b}{'start'} ;
						 my $B_end = $track->{$read_b}{'end'};
						 if (($tig_a == $tig_b) && (defined $tig_info->{$tig_a}{'tot_w'}) ){ ###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
						  $du++;
							if( ($du % 100000)==0 ){
							   #print "shadow pattern du $du du1 $du1\n";
							   #store \%{$tig_info}, "info_$du"; 
							}
							  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
									if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> -> 
										  my $d=$B_start-$A_start-$insert_size;
										  if (abs($d)<($dev_2*$insert_size)){
												   my $w_s; my $w_e;
												   my $dus=1+int($track->{$read_a}{'start'}/$seq_l);
												   if($dus>$tig_info->{$tig_a}{'tot_w'}){  
													  $w_s=$tig_info->{$tig_a}{'tot_w'}; 
													  #print "i told u $tig_info->{$tig_a}{'tot_w'} dus $track->{$read_a}{'start'} $track->{$read_b}{'end'}\n"; 
												   }else{
													  $w_s=$dus;
												   }
												   undef $dus; my $due=1+int($track->{$read_b}{'end'}/$seq_l);
												   if($due>$tig_info->{$tig_a}{'tot_w'}){  
													   $w_e=$tig_info->{$tig_a}{'tot_w'};
												   }else{
													  $w_e=$due;
												   } 
												   for(my $i=$w_s;$i<=$w_e;$i++){
														$tig_info->{$tig_a}{'pilar'}{$i}++; 
												   }
										  }else{
											  $tig_info->{$tig_a}{'in_consis'}++; #  print "tig $tig_a   d $d   B $B_start A $A_start $insert_size\n\n";    
										  }
									}else{     
									   $tig_info->{$tig_a}{'in_consis'}++; #print "ndashtim 0 $tig_a $tig_b $read_a $read_b\n\n\n";
									}       
							  }else{
								  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  <-  <-
										  my $d=$A_start-$B_start-$insert_size;
										  if (abs($d)<($dev_2*$insert_size)){
												   my $w_s; my $w_e;
												   my $dus=1+int($track->{$read_b}{'end'}/$seq_l);
												   if($dus>$tig_info->{$tig_a}{'tot_w'}){  
													  $w_s=$tig_info->{$tig_a}{'tot_w'}; 
													  #print "i told u 2 $tig_info->{$tig_a}{'tot_w'} dus $track->{$read_b}{'end'} $track->{$read_a}{'start'}\n";
												   }else{
													  $w_s=$dus;
												   }
												   undef $dus; my $due=1+int($track->{$read_a}{'start'}/$seq_l);
												   if($due>$tig_info->{$tig_a}{'tot_w'}){  
													  $w_e=$tig_info->{$tig_a}{'tot_w'};
												   }else{
													  $w_e=$due;
												   } 
												   for(my $i=$w_s;$i<=$w_e;$i++){
														$tig_info->{$tig_a}{'pilar'}{$i}++; 
												   }	   
										  }else{
											  $tig_info->{$tig_a}{'in_consis'}++; #  print "tig $tig_a   d $d   A $A_start B $B_start $insert_size\n\n";
										  }
								  
								  }else{     
									   $tig_info->{$tig_a}{'in_consis'}++; #print "ndashtim 1 $tig_a $tig_b $read_a $read_b\n\n\n";
								  }
								  
							  }	  
						 } 
					}else{
						#print "ee ee \n";
					} 
				}     
			 }
		  }
	   }
	   close IN;
   } 	   
   return $tig_info,$num_mate_file,$insert_size;
}
#----------------------------------------------
sub  Empiricalsize{    
   my ($file, $track, $forbiden, $info, $mtig) = @_;
   
   my @aray_in; my @sizes; my $num_self; my $insert_size; my $dev_1; my $dev_2; my $mm=0; my $min_n=200; my $assoc;
 
   my $mean=0; my $N=0;
   foreach my $tig (keys %{$info}){  
        $mean+=$info->{$tig}{'cov'};  $N++; $info->{$tig}{'in_consis'}=0;
   }
   open(IN,$file) || die "Can't open $file -- fatal\n";
   AKHAREMATE: while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
          if ($mm==1) {          
		      print "Associated library: $assoc\n";
			  if ( scalar(@sizes) > 0){
					  @sizes= sort @sizes;   
					  my $L=scalar(@sizes);    

					 my $sum=0;
					 foreach my $t (@sizes){
						   $sum+=$t; 
					  }
					  my $new_i=int ($sum/$L)+1;
					  if ( $L > $min_n ){
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
							 push @aray_in, $new_i;     
					  }else{
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
							 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
							 push @aray_in, $insert_size ;    
					  }
					  
			  }else{
    	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
					push @aray_in, $insert_size ; 
              }
          }
          $mm=1;
          $insert_size=$2; $assoc=$1;

          my $assoc_trim=$1;
          
          if($assoc_trim =~ /(.*)\/([^\/]*)$/){
          	 $assoc_trim=$2;
          
          }else{
          
          } 
          
          open (DDIS, ">$address"."/div/insertsize_distribution_$assoc_trim"."_$insert_size".".txt");
          
          undef @sizes;
          if($insert_size>5000){
               $dev_1=1.1; $dev_2=2;
          }elsif($insert_size>1000){
               $dev_1=1.3; $dev_2=2.5;
          }elsif($insert_size>500){
               $dev_1=1.6; $dev_2=3.25;
          }elsif($insert_size>250){
               $dev_1=1.6; $dev_2=3.75;
          }else{
               $dev_1=1.6; $dev_2=4;
          }                  
          #print "insert_size $insert_size dev_1 $dev_1 dev_2 $dev_2\n";          
      }elsif($line =~ /^Fragment$/){
          if ($mm==1) {          
		      print "Associated library: $assoc\n";
			  if ( scalar(@sizes) > 0){
					  @sizes= sort @sizes;   
					  my $L=scalar(@sizes);    

					 my $sum=0;
					 foreach my $t (@sizes){
						   $sum+=$t; 
					  }
					  my $new_i=int ($sum/$L)+1;
					  if ( $L > $min_n ){
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
							 push @aray_in, $new_i;     
					  }else{
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
							 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
							 push @aray_in, $insert_size ;    
					  }
					  
			  }else{
    	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
					push @aray_in, $insert_size ; 
              }
          }
          $mm=0;         
          last AKHAREMATE;     
      }elsif($line =~ /^([^\>]*)$/i){  
         my $sdna = $1;  
         if($sdna =~/^([ACGT]*)\:([ACGT]*)$/i){
            my $read_a=$1; my $read_b=$2; 
            
            if(defined $track->{$read_a}{'tig'} && $track->{$read_b}{'tig'}){### both pairs assembled
				 my $tig_a = $track->{$read_a}{'tig'};
				 my $tig_b = $track->{$read_b}{'tig'};    
				 my $A_length = $info->{$tig_a}{'length'};
				 my $A_start = $track->{$read_a}{'start'};
				 my $A_end = $track->{$read_a}{'end'};
				 my $B_length = $info->{$tig_b}{'length'};
				 my $B_start = $track->{$read_b}{'start'} ;
				 my $B_end = $track->{$read_b}{'end'};
				 #if(($A_length>100) && ($B_length>100)){		  
					 if($tig_a != $tig_b){####paired reads located on <> contigs
					 }else{###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
					     if( $A_length > (2*$insert_size) ){				  
							  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
									if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> -> 
										  
										  my $imp= $B_start-$A_start; print DDIS "$imp ";
										  my $d=$B_start-$A_start-$insert_size;
										  
										  if ( (abs($d)<($dev_2*$insert_size)) && ($imp>0)){
											   push @sizes, $imp; 
										  }else{
											   
										  }
									}else{     
									   
									}
							  
							  }else{
								  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  <-  <-
									   
										  my $imp= $A_start-$B_start; print DDIS "$imp ";
										  my $d=$A_start-$B_start-$insert_size;
										  
										  if ( (abs($d)<($dev_2*$insert_size)) && ($imp>0)){
											   push @sizes, $imp;  
										  }else{
											   
										  }
								  
								  }else{     
									   
								  }
								  
							  }	  
						 }	  
							  
					 } 
				 #} 
		    }     
         }
      }
   }
   if ($mm==1) {          
		  print "Associated library: $assoc\n";
		  if ( scalar(@sizes) > 0){
				  @sizes= sort @sizes;   
				  my $L=scalar(@sizes);    

				 my $sum=0;
				 foreach my $t (@sizes){
					   $sum+=$t; 
				  }
				  my $new_i=int ($sum/$L)+1;
				  if ( $L > $min_n ){
						 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
						 push @aray_in, $new_i;     
				  }else{
						 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
						 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
						 push @aray_in, $insert_size ;    
				  }
				  
		  }else{
   	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
   	  	   	    push @aray_in, $insert_size ; 
          }
   }      
   close IN; close DDIS;  print "\n";
   return @aray_in;
}
#-----------------------
sub OrientationDistance{    
   my ($file, $track, $forbiden, $info, $mtig, @aray_in) = @_;
   my $ORI_DIS; my $num_self; my $insert_size; my $dev_1; my $dev_2;
   
   my $mean=0; my $N=0;
   foreach my $tig (keys %{$info}){  
        $mean+=$info->{$tig}{'cov'};  $N++; $info->{$tig}{'in_consis'}=0;
   }
   $mean=int($mean/$N); my $cut=2*$mean; #print "for ori-dis nmtig $mtig mean_cov $mean cut $cut\n";
   for(my $tig=1;$tig<$mtig;$tig++){
       if(($info->{$tig}{'length'}>(2*$insert_size)) && ($info->{$tig}{'cov'}<$cut) ){
           my $tot_window=int($info->{$tig}{'length'}/$seq_l);
           $info->{$tig}{'tot_w'}=$tot_window; 
           for(my $i=1;$i<=$info->{$tig}{'tot_w'};$i++){
               $info->{$tig}{'pilar'}{$i}=0; 
           }
       }
   }
   
   open(IN,$file) || die "Can't open $file -- fatal\n";
   AKHAREMATE: while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
           if ($ign_empi==1){
           		$insert_size = $2;
           }else{
		        $insert_size = shift(@aray_in);   
           }
		  print "Associated library: $1\n";
          print "Suggested insert_size: $2         We will use: $insert_size\n\n";
          if($insert_size>5000){
               $dev_1=1.1; $dev_2=.2;
          }elsif($insert_size>1000){
               $dev_1=1.3; $dev_2=.5;
          }else{
               $dev_1=1.6; $dev_2=1;
          }          
          #print "insert_size $insert_size dev_1 $dev_1 dev_2 $dev_2\n";          
      }elsif($line =~ /^Fragment$/){
          last AKHAREMATE;     
      }elsif($line =~ /^([^\>]*)$/i){  
         my $sdna = $1;
         if($sdna =~/^([ACGT]*)\:([ACGT]*)$/i){   
            my $read_a = $1; my $read_b = $2;
            
            if(defined $track->{$read_a}{'tig'} && $track->{$read_b}{'tig'}){### both pairs assembled
				 my $tig_a = $track->{$read_a}{'tig'};
				 my $tig_b = $track->{$read_b}{'tig'};    
				 my $A_length = $info->{$tig_a}{'length'};
				 my $A_start = $track->{$read_a}{'start'};
				 my $A_end = $track->{$read_a}{'end'};
				 my $B_length = $info->{$tig_b}{'length'};
				 my $B_start = $track->{$read_b}{'start'} ;
				 my $B_end = $track->{$read_b}{'end'};
				 if($tig_a != $tig_b){####paired reads located on <> contigs
				    if(! defined $forbiden->{$tig_a}{$tig_b}){
						if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
							  if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> -> 
								 my $d=$insert_size+$A_start-$B_start;
				  
								 my $gap=$d-$info->{$tig_a}{'length'};
								 if(abs($gap)<($dev_1*$insert_size)){
									 if ($tig_a<$tig_b){
										$ORI_DIS->{$tig_a}{$tig_b}{'P'}++;
										$ORI_DIS->{$tig_a}{$tig_b}{'PD'}+=$d;
									 }else{    
										$ORI_DIS->{$tig_b}{$tig_a}{'P'}++;
										$ORI_DIS->{$tig_b}{$tig_a}{'PD'}-=$d;
									 }
								  }else{
								     $kharab1++;
								  }
							  }else{# -> <-
								 my $d=$insert_size+$A_start+$B_start;
								 						 
								 my $gap=$d-$info->{$tig_a}{'length'}-$info->{$tig_b}{'length'};
								 if(abs($gap)<($dev_1*$insert_size)){
									 if ($tig_a<$tig_b){
										$ORI_DIS->{$tig_a}{$tig_b}{'N'}++;
										$ORI_DIS->{$tig_a}{$tig_b}{'ND'}+=$d;
									 }else{
										$ORI_DIS->{$tig_b}{$tig_a}{'N'}++;
										$ORI_DIS->{$tig_b}{$tig_a}{'ND'}+=$d;
									 }
                                 }else{
                                     $kharab2++;
                                 }
							  }
						}else{
							  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  <-  <-
								 my $d=-($insert_size-$A_start+$B_start);	
				  
								 my $gap=-$d-$info->{$tig_b}{'length'};
								 if(abs($gap)<($dev_1*$insert_size)){
									 if ($tig_a<$tig_b){
										$ORI_DIS->{$tig_a}{$tig_b}{'P'}++;
										$ORI_DIS->{$tig_a}{$tig_b}{'PD'}+=$d;
									 }else{
										$ORI_DIS->{$tig_b}{$tig_a}{'P'}++;
										$ORI_DIS->{$tig_b}{$tig_a}{'PD'}-=$d;
									 }
								 }else{
								     $kharab3++
								 }
							  }else{    #   <- ->  
								 my $d=-($insert_size-$A_start-$B_start);
								 
								 my $gap=-$d;
								 if(abs($gap)<($dev_1*$insert_size)){
									 if ($tig_a<$tig_b){
										$ORI_DIS->{$tig_a}{$tig_b}{'N'}++;
										$ORI_DIS->{$tig_a}{$tig_b}{'ND'}+=$d;
									 }else{
										$ORI_DIS->{$tig_b}{$tig_a}{'N'}++;
										$ORI_DIS->{$tig_b}{$tig_a}{'ND'}+=$d;
									 }
								 }else{
								     $kharab4++
								 }									 
							  }
						}
					}else{
					
					}
				 }else{###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
	                  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
	                        if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> -> 
	                        my $du= $B_start-$A_start; 
	                              my $d=$B_start-$A_start-$insert_size;
	                              if (abs($d)<($dev_2*$insert_size)){
	                                   $num_self++;
	                                   if(defined $info->{$tig_a}{'tot_w'}){ 
										   my $w_s; my $w_e;
										   my $dus=1+int($track->{$read_a}{'start'}/$seq_l);
										   if($dus>$info->{$tig_a}{'tot_w'}){  
											  $w_s=$info->{$tig_a}{'tot_w'}; 
										   }else{
											  $w_s=$dus;
										   }
										   undef $dus; my $due=1+int($track->{$read_b}{'end'}/$seq_l);
										   if($due>$info->{$tig_a}{'tot_w'}){  
											   $w_e=$info->{$tig_a}{'tot_w'};
										   }else{
											  $w_e=$due;
										   } 
										   for(my $i=$w_s;$i<=$w_e;$i++){
												$info->{$tig_a}{'pilar'}{$i}++; 
										   }
	                                   }
	                              }else{
	                                  $info->{$tig_a}{'in_consis'}++; 
	                              }
	                        }else{     
	                           $info->{$tig_a}{'in_consis'}++; 
	                        }
	                  
	                  }else{
						  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  <-  <-
						          my $d=$A_start-$B_start-$insert_size;
	                              if (abs($d)<($dev_2*$insert_size)){
	                                   $num_self++;
	                                   if(defined $info->{$tig_a}{'tot_w'}){ 
										   my $w_s; my $w_e;
										   my $dus=1+int($track->{$read_b}{'end'}/$seq_l);
										   if($dus>$info->{$tig_a}{'tot_w'}){  
											  $w_s=$info->{$tig_a}{'tot_w'}; 
										   }else{
											  $w_s=$dus;
										   }
										   undef $dus; my $due=1+int($track->{$read_a}{'start'}/$seq_l);
										   if($due>$info->{$tig_a}{'tot_w'}){  
											  $w_e=$info->{$tig_a}{'tot_w'};
										   }else{
											  $w_e=$due;
										   } 
										   for(my $i=$w_s;$i<=$w_e;$i++){
												$info->{$tig_a}{'pilar'}{$i}++; 
										   }
									   }	   
	                              }else{
	                                  $info->{$tig_a}{'in_consis'}++; 
	                              }
						  
						  }else{     
	                           $info->{$tig_a}{'in_consis'}++;
	                      }
						  
					  }	  
				 } 
		    }     
         }
      }
   }
   close IN;  
   return $ORI_DIS, $num_self, $info;
}
#-----------------------
sub reverseComplement{
   $_ = shift;
   $_ = uc();
   tr/ATGC/TACG/;
   return (reverse());
}
#-----------------  
sub readFasta{
   my ($set,$bin,$file) = @_;
   my $ctrd = 0;
   my ($mate,$frag,$s_l_u,$s_l_d)=(0,0,0,0);
   
   open(IN,$file) || die "Can't open $file -- fatal\n";
   while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
          $mate=1; $frag=0;
      }elsif($line =~ /^Fragment$/){
          $mate=0; $frag=1;       
      }elsif($line =~ /^([^\>]*)$/i){
          if($mate==1){
				if($line =~/^([ACGT]*)\:([ACGT]*)$/i){
				   ($set,$bin,$ctrd) = &loadSequence($set,$bin,$ctrd,$1);
				   ($set,$bin,$ctrd) = &loadSequence($set,$bin,$ctrd,$2);
				   if( $s_l_d == 0 ){
				       $s_l_d=length($1);
				   }elsif( length($1) < $s_l_d ){
				       $s_l_d=length($1);
				   }
				   if( length($1) > $s_l_u ){
				       $s_l_u=length($1);
				   }
				}else{
				   my $pairing_failure_message = "The sequence \"$line\" is not in the right format for paired-end reads  -- Fatal\nMake sure your input is in the form:\n\n>test\nGCTACGACTATGACATACAGT:GTAGATTGATCGCATGCACGCT\n\nWhere : separates paired reads.  Spaces in your input file might have caused this error.\n";
				   print $pairing_failure_message;
				   exit;
				}            
          }else{
				if ($line =~ /^([ACGT]*)$/i){ 
					($set,$bin,$ctrd) = &loadSequence($set,$bin,$ctrd,$1);
				    if( $s_l_d == 0 ){
				       $s_l_d=length($1);
				    }elsif( length($1) < $s_l_d ){
				       $s_l_d=length($1);
				    }
				    if( length($1) > $s_l_u ){
				       $s_l_u=length($1);
				    }				    
				}else{
				   my $failure_message = "The sequence \"$line\" is not in the right format -- Fatal\nMake sure your input is in the form:\n\n>test\nGCTACGACTATGACATACAGT\n\nSpaces in your input file might have caused this error.\n";
				   print $failure_message;
				   exit;
				}        
          }
      }
   }
   close IN;
   my $read_number_message = "$ctrd total sequences (" . keys( %$set ) . " unique)\n";
   printf $read_number_message;
   return $set,$bin,$s_l_u,$s_l_d;
}
#-----------------  
### added 31Jan08 R.Warren
sub loadSeed{
   my $file = shift;
   my $seed;  
   open(IN,$file) || die "Can't open $file -- fatal\n";
   my ($subseq,$prev)=('','');
   while(<IN>){
      chomp;
      if (/\>(\S+)/){
         my $head=$1;
         my $subseq_length = length($subseq);
         if($head ne $prev && $subseq ne '' && $subseq_length >= $MIN_READ_LENGTH && $subseq_length >= $min_overlap){
            $seed->{$subseq}{'count'}++;
            $seed->{$subseq}{'name'} = $prev;
            if($subseq=~/([NX])/i){print "WARNING: the fasta sequence >$prev in your seed file contains characters other than ACGT (i.e. $1) and may prevent proper contig extension.\n";}
         }
         $subseq='';
         $prev=$head;
      }elsif(/^([ACGTNX]*)$/i){
         $subseq .= uc($_);
      }
   }
   my $subseq_length = length($subseq);
   if($subseq ne '' && $subseq_length >= $MIN_READ_LENGTH && $subseq_length >= $min_overlap){
      $seed->{$subseq}{'count'}++;
      $seed->{$subseq}{'name'} = $prev;
      if($subseq=~/([NX])/i){print "WARNING: the fasta sequence >$prev in your seed file contains characters other than ACGT (i.e. $1) and may prevent proper contig extension.\n";}
   }
   close IN; return $seed;
}
#-----------------
sub loadSequence{
   my ($set,$bin,$ctrd,$seq) = @_;
   my $orig=uc($seq);
   my $orig_mer = length($orig);
   if ($orig ne '' && $orig_mer >= $MIN_READ_LENGTH && $orig_mer >= $min_overlap){
      ####show progress
      my $s10k = $ctrd / 10000;
      print "." if ($s10k == int($s10k) && $s10k && $ctrd);
      $|=1; ###clear buffer
      my $s100k = $ctrd / 100000;
      printf "%i sequences inputted\n", ($ctrd) if ($s100k == int($s100k) && $s100k && $ctrd);
      my @f=split(//,$orig);
      $set->{$orig}{'count'}++;
      my $rc=reverseComplement($orig);
      my @r=split(//,$rc);
      ### added 31Jan08 R.Warren
      $MAX=$orig_mer if ($orig_mer > $MAX);
      $bin->{$f[0]}{$f[1]}{$f[2]}{$f[3]}{$f[4]}{$f[5]}{$f[6]}{$f[7]}{$f[8]}{$f[9]}{$f[10]}{$orig}++;
      $bin->{$r[0]}{$r[1]}{$r[2]}{$r[3]}{$r[4]}{$r[5]}{$r[6]}{$r[7]}{$r[8]}{$r[9]}{$r[10]}{$rc}++;
      $ctrd++;
   }elsif($orig ne ''){
      if($orig_mer < $MIN_READ_LENGTH){
         print SHO "$seq\tInput sequence shorter than minimum read length allowed ($orig_mer < $MIN_READ_LENGTH nt)\n";
      }elsif($orig_mer < $min_overlap){
         print SHO "$seq\tInput sequence shorter than minimum overlap specified($orig_mer < -m $min_overlap)\n";
      }
   }
   $MAX = $MAX_TOP if ($MAX > $MAX_TOP);
   return $set,$bin,$ctrd;
}
#------------------------------------
sub readFasta_forc{
   my ($set,$file) = @_;
   my $ctrd = 0;
   my ($mate,$frag)=(0,0);
   
   open(IN,$file) || die "Can't open $file -- fatal\n";
   while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
          $mate=1; $frag=0;
      }elsif($line =~ /^Fragment$/){
          $mate=0; $frag=1;       
      }elsif($line =~ /^([^\>]*)$/i){
          if($mate==1){
				if($line =~/^([ACGT]*)\:([ACGT]*)$/i){
				   $set = &loadSequence_forc($set,$1);
				   $set = &loadSequence_forc($set,$2);

				}else{
				}            
          }else{
				if ($line =~ /^([ACGT]*)$/i){ 
					$set = &loadSequence_forc($set,$1);
			
				}else{
				}        
          }
      }
   }
   close IN;
   return $set;
}
#------------------------------------
sub loadSequence_forc{
   my ($set,$seq) = @_;
   my $orig=uc($seq);
   my $orig_mer = length($orig);
   if ($orig ne '' && $orig_mer >= $MIN_READ_LENGTH && $orig_mer >= $min_overlap){
      $|=1;
      $set->{$orig}{'count'}++;
   }
   return $set;
}
#------------------------------------
sub trackReads{
   my ($track, $track_all) = @_;
   foreach my $rd (keys %$track){
      if(! defined $track_all->{$rd}{'tig'}){
         $track_all->{$rd}{'tig'} = $track->{$rd}{'tig'};
         $track_all->{$rd}{'start'} = $track->{$rd}{'start'};
         $track_all->{$rd}{'end'} = $track->{$rd}{'end'};
         delete $track->{$rd};
      }
   }
   return $track_all;
}
#------------------------------------
sub getDistance{

   my ($insert_size, $length_i, $start_i, $start_j) = @_;

   # L  ------  --------- R
   # i    ->        <-    j
   #      ....  ......    insert_span
   #      ============    insert_size

   my $insert_span = ($length_i - $start_i) + $start_j;
   my $gap_or_overlap = $insert_size - $insert_span;

   return $gap_or_overlap;
}
#-----------------
# SSAKE contig extension
sub doExtension{

   my ($direction, $orig_mer, $seq, $set, $bin, $reads_needed, $total_bases, $min_overlap, $base_overlap, $min_base_ratio, $verbose, $track, $tig_count, $max_trim, $alon, @c_n) = @_;

   my $previous = $seq;
   my $extended = 1;
   my $trim_ct = 0;     #trim counter - keeps track of 3'-end trim

   TRIM:
   while($trim_ct <= $max_trim){

      while($extended){

         ### added 31Jan08 R.Warren
         if($orig_mer > $MAX){$orig_mer=$MAX;}  ### Deals with special cases where the seed sequences are different from the read set (and possibly very large) - goal here is not to increase sequence coverage of seed, but rather to extend it.

         my ($ct,$pos,$current_reads,$current_bases,$span) = (0,0,0,0,$orig_mer);

         my $overhang = {};
         my @overlapping_reads = ();
         for (my $x=1;$x <= ($orig_mer *2);$x++){      #why , what is $orig_mer *2
            ($overhang->{$x}{'A'},$overhang->{$x}{'C'},$overhang->{$x}{'G'},$overhang->{$x}{'T'}) = (0,0,0,0);            
         }

         EXTENSION:
         while ($span > $min_overlap){  #will slide the subseq, until the user-defined min overlap size

            ### Added 19March08
            if(length($seq) >= $MAX){   # $seq is length of contig being extended -- if larger than largest read, make sure the largest read could align and all subsequent rds. 
               $span = $MAX - $ct;
            }else{
               $span = length($seq) - $ct;
            }

            $pos = length($seq) - $span;

            my $subseq = substr($seq, $pos, $span);                                                            #make a sub-sequence of length l-(1..i) for searching

            my @s=split(//,$subseq);
            my $subset = $bin->{$s[0]}{$s[1]}{$s[2]}{$s[3]}{$s[4]}{$s[5]}{$s[6]}{$s[7]}{$s[8]}{$s[9]}{$s[10]}; #Will grab everything even the reverse complement ones
           
            print "####$direction' SEARCH Counter:$ct Position:$pos Span:$span - Subseq:$subseq Previous:$previous\n" if ($verbose);
            
            SEARCH:   #this cycles through limited k-mer space
            foreach my $pass (sort {$subset->{$b} <=> $subset->{$a}} keys %$subset){    
               if($pass =~ /^$subseq([ACGT]*)/){ 
                  #can we align perfectly that subseq to another rd start?
                  my $dangle = $1;
                  print "\n", "=" x 80, "\n$direction'- FOUND sequence: $pass -> subset: $subseq -> overhang: $dangle\n", "=" x 80, "\n\n" if ($verbose);

                  # Collect all overhangs
                  push @overlapping_reads, $pass;                  ### all overlapping reads
                  my @over = split(//,$dangle);
                  my $ct_oh = 0;
 
                  foreach my $bz(@over){
                     $ct_oh++;                                     ### tracks overhang position passed the seed  
                     if(defined $set->{$pass}){
                        $overhang->{$ct_oh}{$bz} += $set->{$pass}{'count'};      ### reflects read coverage (often real duplicates)
                     }else{
                        my $pass_rc = reverseComplement($pass);
                        $overhang->{$ct_oh}{$bz} += $set->{$pass_rc}{'count'};
                     } 
                     print "$ct_oh - $bz = $overhang->{$ct_oh}{$bz}\n" if($verbose);
                  }
               }elsif($subseq =~ /$pass/){ ###cases where the read is fully embedded in the search sequence - want to include for coverage calculations
                  my $complement_pass = reverseComplement($pass);

                  print "$pass found in $subseq ($set->{$pass}{'count'}) - deleting read: $pass and complement ($set->{$complement_pass}): $complement_pass\n\n" if ($verbose);
                  my $tcount;
                  if(defined $set->{$pass}){
                        $tcount+=$set->{$pass}{'count'}; 
                  }
                  if(defined $set->{$complement_pass}){
                        $tcount+=$set->{$complement_pass}{'count'};
                  }
                  push @c_n, $tcount;
                  if(defined $set->{$pass}){
                     $current_reads = $set->{$pass}{'count'};
                     $current_bases = length($pass) * $current_reads;
                     $reads_needed += $current_reads;
                     $total_bases += $current_bases;
                     ($bin,$set,$seed) = deleteData($bin,$set,$seed,$pass);
                     if(! defined $track->{$pass}){
                         if($tcount<$copy_li){
							if ($direction=='3'){
							   $track->{$pass}{'tig'} = $tig_count;
							   $track->{$pass}{'start'} = $pos + 1;
							   $track->{$pass}{'end'} = $pos + length($pass);
							}else{
							   $track->{$pass}{'tig'} = $tig_count;
							   $track->{$pass}{'start'} = $pos ;
							   $track->{$pass}{'end'} = $pos + length($pass)-1;
							}
						 }else{
						    $kharab5++; 
						 }
                     }
                  }
                  if(defined $set->{$complement_pass}){
                     $current_reads = $set->{$complement_pass}{'count'};
                     $current_bases = length($complement_pass) * $current_reads;
                     $reads_needed += $current_reads;
                     $total_bases += $current_bases;
                     ($bin,$set,$seed) = deleteData($bin,$set,$seed,$complement_pass);
                     if(! defined $track->{$complement_pass}){ 
                        if($tcount<$copy_li){
							if ($direction=='3'){   
							   $track->{$complement_pass}{'tig'} = $tig_count;
							   $track->{$complement_pass}{'end'} = $pos + 1;
							   $track->{$complement_pass}{'start'} = $pos + length($complement_pass);
							}else{
							   $track->{$complement_pass}{'tig'} = $tig_count;
							   $track->{$complement_pass}{'end'} = $pos;
							   $track->{$complement_pass}{'start'} = $pos + length($complement_pass)-1;
							}
					    }else{
					        $kharab5++;
					    }
                     }
                  }
               }  
            }
            $ct += $SEQ_SLIDE;
         }#while overlap >= user-defined -m minimum
 
         my $consensus = "";
         print "Finished Collecting Overlapping Reads - BUILDING CONSENSUS...\n" if ($verbose);
         print Dumper(@overlapping_reads) if ($verbose);

         ### Build consensus
         CONSENSUS:
         foreach my $ohpos (sort {$a<=>$b} keys %$overhang){
            if($ohpos){

               my $coverage = $overhang->{$ohpos}{'A'}+$overhang->{$ohpos}{'C'}+$overhang->{$ohpos}{'G'}+$overhang->{$ohpos}{'T'};             
               print "pos:$ohpos cov:$coverage A:$overhang->{$ohpos}{'A'} C:$overhang->{$ohpos}{'C'} G:$overhang->{$ohpos}{'G'} T:$overhang->{$ohpos}{'T'}\n" if($verbose);

               if ($coverage < $base_overlap){
                  print "COVERAGE BELOW THRESHOLD: $coverage < -o $base_overlap @ $ohpos :: will extend by: $consensus\n" if ($verbose);
                  last CONSENSUS;
               }
               my $baselist = $overhang->{$ohpos};

               my $ct_dna=0;
               my $previous_bz = "";

               BASE:
               foreach my $bz (sort {$baselist->{$b}<=>$baselist->{$a}} keys %$baselist){
                  #print "\t$ct_dna -> $bz..$baselist->{$previous_bz} > $baselist->{$bz}\n";
                  if($ct_dna){## the two most abundant bases at that position
                     #print "\t\t$ct_dna\n";
                     if($previous_bz ne "" && ($baselist->{$previous_bz} / $coverage) >= $min_base_ratio && $baselist->{$previous_bz} > $baselist->{$bz}){### a simple consensus btw top 2 
                        $consensus .= $previous_bz;                                         ### build consensus
                        print "Added base $previous_bz (cov = $baselist->{$previous_bz}) to $consensus **\n" if ($verbose);
                        last BASE;
                     }else{
                        print "ISSUES EXTENDING: best base = $previous_bz (cov=$baselist->{$previous_bz}) at $ohpos.  Second-Best: $bz (cov=$baselist->{$bz}) (ratio best=$baselist->{$previous_bz} / total=$coverage) >= $min_base_ratio (-r) -- will terminate with $consensus\n" if($verbose);
                        last CONSENSUS;
                     }
                  }
                  $previous_bz = $bz;                 
                  $ct_dna++;
               }
            }
         }

         ### deal with sequence reads making up the consensus/newly formed contig
         if($consensus ne ""){
            print "Will extend $seq\nwith: $consensus\n\n" if($verbose);
            my $temp_sequence = $seq . $consensus;  ## this is the contig extension
            my $integral = 0;
            foreach my $ro (@overlapping_reads){

               while($temp_sequence =~ /$ro/g){                                   ### read found integral in the newly built sequence
                  $alon=0;
                  my $complement_ro = reverseComplement($ro);
                  $integral=1;
                  my $tcount;
                  if(defined $set->{$ro}){
                        $tcount+=$set->{$ro}{'count'}; 
                  }
                  if(defined $set->{$complement_ro}){
                        $tcount+=$set->{$complement_ro}{'count'};
                  }                  
                  push @c_n, $tcount;
                  print "$ro found in $seq ($set->{$ro}{'count'}) - deleting read: $ro and complement ($set->{$complement_ro}{'count'}): $complement_ro\n\n" if ($verbose); 
                  if(defined $set->{$ro}){     
                     $current_reads = $set->{$ro}{'count'};  
                     #print "fwd SET:$current_reads BIN $subset->{$ro}\n";
                     $current_bases = length($ro) * $current_reads;
                     $reads_needed += $current_reads;
                     $total_bases += $current_bases;

                     if(! defined $track->{$ro}){
                         if($tcount<$copy_li){
							if ($direction=='3'){
							   $track->{$ro}{'tig'} = $tig_count;
							   $track->{$ro}{'start'} = (pos $temp_sequence) - length($ro) + 1;
							   $track->{$ro}{'end'} =  (pos $temp_sequence);
							}else{ 
							   $track->{$ro}{'tig'} = $tig_count;
							   $track->{$ro}{'start'} = (pos $temp_sequence) - length($ro) ;
							   $track->{$ro}{'end'} =  (pos $temp_sequence)-1;
							}
						 }else{
						       $kharab5++;
						 }
                     }       
                     ($bin,$set,$seed) = deleteData($bin,$set,$seed,$ro);
                  }

                  if(defined $set->{$complement_ro}){  
                     $current_reads = $set->{$complement_ro}{'count'}; 
                     #print "rc SET:$current_reads BIN $subset_rc->{$complement_ro}\n";
                     $current_bases = length($complement_ro) * $current_reads;
                     $reads_needed += $current_reads;
                     $total_bases += $current_bases;
              
                     if(! defined $track->{$complement_ro}){
                         if($tcount<$copy_li){
							if ($direction=='3'){                     
							   $track->{$complement_ro}{'tig'} = $tig_count;
							   $track->{$complement_ro}{'end'} = (pos $temp_sequence) - length($complement_ro) + 1;
							   $track->{$complement_ro}{'start'} = (pos $temp_sequence);
							}else{  
							   $track->{$complement_ro}{'tig'} = $tig_count;
							   $track->{$complement_ro}{'end'} = (pos $temp_sequence) - length($complement_ro);
							   $track->{$complement_ro}{'start'} = (pos $temp_sequence)-1;
							}
						 }else{
						       $kharab5++;
						 }
                     }
                     ($bin,$set,$seed) = deleteData($bin,$set,$seed,$complement_ro);
                  }
               }
            }
            if(! $integral){### no reads are found overlapping with the consensus might be indicative of low complexity regions -- Stop the extension
               print "No overlapping reads agree with the consensus sequence.   Stopping extension" if ($verbose);
               $extended = 0;
            }else{
               $seq = $temp_sequence;
               print "New Contig is: $seq\n" if ($verbose);
               $extended = 1;
            }
            $previous = $seq;
         }else{### no consensus built, will stop the extension
            $extended = 0;
         }
      }###while get the OK for extension
      $trim_ct++;
      if ($trim_ct <= $max_trim){
         last TRIM if (length($seq) <= $MIN_READ_LENGTH); #terminate assembly if trimming becomes too agressive
         $seq = substr($seq, 0, -1);
         $extended = 1;
         print "\n$direction prime EXTENSION ROUND $trim_ct COMPLETE UNTIL $max_trim nt TRIMMED OFF => TRIMMED SEQUENCE:$seq\n\n" if ($verbose);
      }
   }### while trimming within bounds
   #### Adjust the position if tracking paired reads in assembly
   if($paired){
	   foreach my $rd (keys %$track){
		  $track->{$rd}{'start'} = length($seq) - $track->{$rd}{'start'};
		  $track->{$rd}{'end'} = length($seq) - $track->{$rd}{'end'};
	   }
   } 	   
   print "\n*** NOTHING ELSE TO BE DONE IN $direction prime- PERHAPS YOU COULD DECREASE THE MINIMUM OVERLAP -m (currently set to -m $min_overlap) ***\n\n" if ($verbose);
   return $seq, $set, $bin, $reads_needed, $total_bases, $track, $alon, @c_n;
}
#-----------------------
sub deleteData {
   my ($bin,$set,$seed,$sequence) = @_;
   my @o=split(//,$sequence);
   my $comp_seq = reverseComplement($sequence);
   my @c=split(//,$comp_seq);
   #remove k-mer from hash table and prefix tree
   delete $bin->{$o[0]}{$o[1]}{$o[2]}{$o[3]}{$o[4]}{$o[5]}{$o[6]}{$o[7]}{$o[8]}{$o[9]}{$o[10]}{$sequence};
   delete $bin->{$c[0]}{$c[1]}{$c[2]}{$c[3]}{$c[4]}{$c[5]}{$c[6]}{$c[7]}{$c[8]}{$c[9]}{$c[10]}{$comp_seq};
   delete $set->{$sequence};
   delete $seed->{$sequence};
   return $bin,$set,$seed;
}
#-----------------------
### We hope this code is useful to you -- Please send comments & suggestions to rwarren * bcgsc.ca
