#! /usr/bin/perl

#AUTHOR
#  Adel Dayarian

#The code is available freely, under the GNU Public License, at http://www.physics.rutgers.edu/~anirvans/SOPRA/

use strict;
use Getopt::Long;

my @fragment_names;
my @f3_names;
my @r3_names;
my @distances;
my $address="";

&GetOptions( "frag=s{,}" => \@fragment_names,
             "f3=s",  => \@f3_names,
     	     "r3=s",  => \@r3_names,
     	     "d=i",  => \@distances,
     	     "a=s", => \$address,
           ) or die "Unknown option\n";
#-------------------------------------------------   
if( ((! @f3_names) && (! @r3_names) && (! @fragment_names)) || ( $address eq "" ) ){
   print "Usage: $0\n\n";
   print "-f3  File in fasta format containing F3 reads for a mate pair library\n";
   print "-r3  File in fasta format containing R3 reads for the corresponding mate pair library\n";
   print "-d  The insert size for the corresponding mate pair library\n\n"; 
   print "-frag  File in fasta format containing reads for a fragment (unpaired) library\n\n";
   print "-a  Name of the output directory\n\n";   
   
   print "example: $0 -f3 mate1_F3.fasta -r3 mate1_R3.fasta -d L1 -f3 mate2_F3.fasta -r3 mate2_R3.fasta -d L2 -frag fragment1.fasta fragment2.fasta -a mydir_sout\n\n";
   
   exit;
}

if( $address =~ /^(.*)\/$/ ){
     $address=$1; 
}

print "Fragment files:\n@fragment_names\n";
print "f3 files:\n@f3_names\n";
print "r3 files:\n@r3_names\n";
print "Inser sizes:\n@distances\n";
print "Output directory:\n$address\n";

if ( scalar(@f3_names)==0 ){
   print "\nNo mate-pair file specified\n";
}
if ( scalar(@f3_names)!=scalar(@r3_names) ){
   die "Number of f3 files is not equal to the number of r3 files\n";
}
if ( ( scalar(@f3_names) > 0 ) && ( scalar(@f3_names)!=scalar(@distances) ) ){
   die "You specified that your input consists of mate pairs, but did not specify valid insert length (use -d option)\n";
}

if ( ($address ne "." ) && ($address ne "" ) ){     
	 mkdir("$address", 0777) || print $!;
	 $address=$address."/sread_in.fasta";
}else{
	 $address=$address."/sread_in.fasta";
}

open (OUT,">$address") or die  "$!";  

my $tot_seq; my $num_seq;
my $num_matefile=scalar(@f3_names)-1;

for (my $i=0; $i <= $num_matefile ; $i++){
    $num_seq=0;
    
    open (F3_FILE,"<$f3_names[$i]") or die "Cannot open $f3_names[$i] $!\n";
    open (R3_FILE,"<$r3_names[$i]") or die "Cannot open $r3_names[$i] $!\n";

    if( $distances[$i] > 0 ){
        print OUT "Mate.$f3_names[$i]"."_$distances[$i]\n";
    }else{
        die "Invalid insert length $distances[$i]\n";
    }
    
    while (my $line_f3=<F3_FILE>){
        if( $line_f3 =~ /^\#/ ){
            
            my $line_r3=<R3_FILE>; 
            
            if( $line_r3 =~ /^\#/ ){ 
            
            }else{
                die "$r3_names[$i] is not in the right format\n";
            }        
            
        }elsif( $line_f3 =~ /^\>/ ){
           
            print OUT "$line_f3";
            
            $line_f3=<F3_FILE>; chomp($line_f3);
            
            my $line_r3=<R3_FILE>; 
            
            if( $line_r3 =~ /^\>/ ){
				$line_r3=<R3_FILE>; 				
				print OUT "$line_f3".":$line_r3";
            }else{
                die "$r3_names[$i] is not in the right format\n";
            }
            $tot_seq+=2; $num_seq+=2;
        }else{
            die "$f3_names[$i] is not in the right format\n";
        }
    }   
    close F3_FILE; close R3_FILE;
    print "\n$f3_names[$i] and $r3_names[$i]        number of reads: $num_seq          insert length: $distances[$i]\n";
}

my $num_fragmentfile=scalar(@fragment_names)-1;

for (my $i=0; $i <= $num_fragmentfile ; $i++){  
    $num_seq=0;

    open (FRAG,"<$fragment_names[$i]") or die "Cannot open $fragment_names[$i] $!\n";

    print OUT "Fragment\n";
    
    while (my $line=<FRAG>){
        if( $line =~ /^\#/ ){
        
        }elsif( $line =~ /^\>/ ){
            print OUT "$line";
            $line=<FRAG>; print OUT "$line";
	
			$tot_seq++; $num_seq++;

        }else{
            die "$fragment_names[$i] is not in the right format\n";
        }  
    }
    close FRAG; 
    print "$fragment_names[$i]      number of reads: $num_seq\n";
}
close OUT;

print "\nTotal number of reads  $tot_seq\n\n";