#!/usr/bin/perl

#AUTHOR
#  Adel Dayarian

#The code is available freely, under the GNU Public License, at http://www.physics.rutgers.edu/~anirvans/SOPRA/

use Storable;
use strict;
use Data::Dumper;
use Getopt::Long;

my $opt_g;
my $opt_z;
my $opt_c;
my $opt_e=0;
my $opt_a;
my $opt_pid;

my $address="";

&GetOptions( "g=s",  => \$opt_g,
     	     "z=i",  => \$opt_z,
     	     "c=i",  => \$opt_c,
     	     "e=i",  => \$opt_e,
     	     "pid=i",  => \$opt_pid,
     	     "a=s", => \$opt_a,
           ) or die "Unknown option\n";
           
my ($pairead_file,$contig_size_cutoff,$s_l_u,$s_l_d,$copy_li,$ign_empi)=('sread_in.fasta',0,0,0,5,0);
#-------------------------------------------------
if( (! $opt_g) || (! $opt_a) ){      
   print "Usage: $0\n";
   print "-g  afg file outputed by Velvet -required (run Velvet in fragment mode, with options: -trkg_read yes -amos_file yes)\n";
   print "-a  The directory that you used for output of the formating script (format_base.pl)\n\n";   
   
   print "-z Minimum contig size (default 1.5*shortest_read_length)\n";
   print "-e  If set equal to 1, the empirical value for the insert size will not be used\n";   
   print "-c  If the number of times a read and its reverse complement appear in the library is equal to or more than this value, the pairing information from that read will be disregarded  (default -c 5)\n\n";
   
   print "example: $0 -g velvet asm.afg -a mydir_sout\n\n";
   
   exit;
}
my $date = `date`; chomp($date); print "\nStart $date\n\n"; 
my $afg_file = $opt_g;
$contig_size_cutoff = $opt_z if ($opt_z);
$copy_li=$opt_c if($opt_c);
if( $opt_e == 1 ){
      $ign_empi=1;
}

$address= $opt_a;      

if( $address =~ /^(.*)\/$/ ){
     $address=$1; 
}
if (-d $address) {
	#print "There is a directory named $address\n";
}else {
	print "There is no such directory $address\n"; exit;
}

$pairead_file=$address."/$pairead_file"; 
 
mkdir("$address"."/div", 0777) || print $!;
#-------------------------------------------------
my $pid_num;
if ( $opt_pid ){
   $pid_num = $opt_pid; 
}else{
   $pid_num = getpgrp(0); 
} 
my $init_message = "\nRunning: $0\n-g $afg_file\n-c $copy_li\n-e $ign_empi\n-a $address\n\n";
print $init_message;
print "pid $pid_num\n\n";  

 my $derazi;
open(IN,$afg_file) or die "Cannot open $afg_file $!\n";
while(<IN>){
    chomp();
    if( $_ =~ /^\{RED/){
       my $line=<IN>; chomp($line);
       if ( $line =~ /^iid/ ){
       
	   }else{

	   }
	   $line=<IN>; $line=<IN>; $line=<IN>; chomp($line);
	   if( $line =~ /^seq/){
	   		 #$line=<IN>; chomp($line);
	   }
	   if( $line =~ /^([ACGT]*)$/){
	       my $read=$line;
	       
	       $line=<IN>; chomp($line);
		   while ( $line =~ /^([ACGT]*)$/){
					 $read.=$line;
					 $line=<IN>; chomp($line);
		   }  
		   
		   if( $s_l_d == 0 ){
			   $s_l_d=length($read);
		   }elsif( length($read) < $s_l_d ){
			   $s_l_d=length($read);
		   }
		   if( length($read) > $s_l_u ){
			   $s_l_u=length($read);
		   }
		   $derazi->{length($read)}++; 
	   }else{

	   }
    }
}
close IN; 
foreach my $l (sort {$a <=> $b} (keys %{$derazi})){
	print "$derazi->{$l} reads with length $l\n";
}

if($contig_size_cutoff==0){
	$contig_size_cutoff=int(1.5*$s_l_d); 
}
print "max read length $s_l_u bp      min read length $s_l_d bp      contig_size_cutoff $contig_size_cutoff\n";

my $seq_l=$s_l_u;
print "\nExtracting afg information\n\n"; 
#-------------------------------------------------

my ($string, $track_all, $track, $whoshere, $bad_pass, $black_pass, $tig_info, $forbiden_link, $Ori_Dis, $selfpair); 
my ($du_badpas, $du_badpas2, $du_badpas3, $du_badpas4, $du_badpas5, $du_badpas6, $du_badpas7, $du_badpas8, $du_badpas9, $kharab1, $kharab2, $kharab3, $kharab4);  

open(IN,$afg_file) or die "Cannot open $afg_file $!\n";

while(<IN>){
	if ( $_ !~ /^\{/ ){
		die "$afg_file is not the afg file or is not the in proper format\n";
	}else{
		last;
	}
}
close(IN);

my ($id_info,$node_info,$node,$read_id,$off,$start,$end,$position); my $state = 0;
open(IN,$afg_file) or die "Cannot open $afg_file $!\n";
while(<IN>){
    chomp;
	if ($state==0){
        	if( $_ =~ /^\{CTG/){
                $state=1;
			    redo;
        	}
	}else{
        if ( $_ =~ /^\{CTG/ ){
			    #print OUT "-end\n";
        }elsif (  $_ =~ /^\{RED/ ){
			$state=0;
			redo;
		}elsif( $_=~/^\{SCF/){
			$state=0;
			redo;
		}elsif( $_ =~ /^eid/ ){
			$node++; 
		}elsif ( $_ =~ /^src/ ){
			$read_id=$_;   
			$read_id=~s/src://;
        }
        if ( $_ =~ /^off/ ){
			$off=$_;  
			$off=~s/off://;
        }
        if( $_ =~ /^clr/ ){
			$position=$_;  
			$position=~s/clr://;
			if($off>=0){	
				if(!defined $id_info->{$read_id}){
					my @pos=split(/,/,$position);
					if($pos[0]<$pos[1]){
						$id_info->{$read_id}{'off'}=$off;  
						if($pos[0]>0){
						   print "problem with the format of the afg file, pos0 $pos[0] pos1 $pos[1] , read_id $read_id\n"; exit;
						}
						if( !defined $derazi->{$pos[1]} ){
						   print "problem with the format of the afg file, pos_0 $pos[0] pos_1 $pos[1] , read_id $read_id\n"; exit; 
						}	
					}else{			
						$id_info->{$read_id}{'off'}=-($off+$pos[0]-1); 
						if($pos[1]>0){
						   print "problem with the format of the afg file, pos1 -- $pos[0] , $pos[1] , read_id $read_id\n"; exit;
						}	
						if( !defined $derazi->{$pos[0]} ){
						   print "problem with the format of the afg file, pos0 -- $pos[0] , $pos[1] , read_id $read_id\n"; exit;
						}	
					}
					$id_info->{$read_id}{'tig'}=$node;
				}else{
				    #print "charaaa id $read_id old off $id_info->{$read_id}{'off'} old tig $id_info->{$read_id}{'tig'} nnew tig $node new off $off\n";
				    #$id_info->{$read_id}{'bad'}++;
				     $du_badpas9++;
				}
            }
        }
  	}
}
close(IN);

mkdir("$address"."/s_tmp", 0777) || print $!;

open(IN,$afg_file) or die "Cannot open $afg_file $!\n";
while(<IN>){
    chomp();
    if( $_ =~ /^\{RED/){
       my $line=<IN>; chomp($line);
       if ( $line =~ /^iid/ ){
			$read_id=$line;
			$read_id=~s/iid://;	
	   }else{
	        print "problem with the format of the afg file $line\n"; exit;
	   }
	   $line=<IN>; $line=<IN>; $line=<IN>; chomp($line);

	   if( $line =~ /^seq/){
	   		 #$line=<IN>; chomp($line);
	   }
	   
	   if( $line =~ /^([ACGT]*)$/){
	       my $one=$line;
	       $line=<IN>; chomp($line);
	       while ( $line =~ /^([ACGT]*)$/){
	             $one.=$line;
	             $line=<IN>; chomp($line);
	       }	     
	        if (defined $id_info->{$read_id}{'tig'}){
			   if(!defined $node_info->{$id_info->{$read_id}{'tig'}}{$one}){                        
					 $node_info->{$id_info->{$read_id}{'tig'}}{$one}=$id_info->{$read_id}{'off'};
					 if(defined $id_info->{$read_id}{'bad'}){
						 $black_pass->{$one}++; $du_badpas8++;
					 }
			   }else{
					 if(($id_info->{$read_id}{'off'}!=$node_info->{$id_info->{$read_id}{'tig'}}{$one}) ){
						  #print "new off $id_info->{$read_id}{'off'} , old off $node_info->{$id_info->{$read_id}{'tig'}}{$one} $one\n";
					 }
			   }
			}   
	   }else{
	       print "problem with the format of the afg file $line\n";
	   }
    }
}
close IN;
print "Writing to temp file\n";
open(TEMP,">$address"."/s_tmp/temp") or die "Cannot write to file in temp directory\n";
foreach my $tig (sort {$a <=> $b} (keys %{$node_info})){
    print TEMP "=NODE$tig\n";
    foreach my $seq (keys %{$node_info->{$tig}}){
        print TEMP ">,$node_info->{$tig}{$seq}\n$seq\n";
    }
}
close(TEMP);
undef $id_info; undef $node_info;
print "Finished writing to temp file\n\n";

#my $date = `date`; chomp($date); print "$date\n";     
open(TEMP,"$address"."/s_tmp/temp") || die "Can't open temp -- fatal\n";

my $base_name .= "_pid" . $pid_num;

open (TIG, ">$address"."/contigs". $base_name .".fasta" ) or die "$!";

$date = `date`; chomp($date); 
print "Contig reconstruction started    $date     pid $pid_num\n";  
$node=0; my @AG = ('A','T','C','G');
while(my $line=<TEMP>){
   chomp $line; 
   if($line=~/^=NODE.*$/){
       if($node>0){
          my @position= sort {$a <=> $b} (keys %{$string});  
          my $L=scalar(@position); my ($l_kill,$r_kill)=(0,0); my $new_cov;
          my $contig;
          my $dus=shift(@position); my $due=pop(@position); my $move=$dus-1;          
		  for (my $i=$dus; $i <= $due ; $i++){
			 if(defined $string->{$i}){
				 my $add;
				 foreach my $col (sort { $string->{$i}{$b} <=> $string->{$i}{$a} } keys %{$string->{$i}}){     
					 $contig.=$col;
					 $add=$col;
					 last;
				 }  
				 $new_cov+=$string->{$i}{$add}; 
				 if(scalar(keys %{$string->{$i}})>1){
				 	 my $coverage=0; 
				 	 foreach my $j (@AG){
				 	 	 if ( defined $string->{$i}{$j} ) {
				 	 	 	 $coverage+=$string->{$i}{$j};
				 	 	 }
				 	 }				 	 
					 my $l_cv=$string->{$i}{$add}/$coverage;
					 if($l_cv>.9){
						 foreach my $pass (keys %{$whoshere->{$i}}){
							  if($whoshere->{$i}{$pass} eq $add){
							  }else{
								  delete $track->{$pass};
							  }
						 }
					 }else{
						 foreach my $pass (keys %{$whoshere->{$i}}){
							 delete $track->{$pass};
						 }					 		 
					 }
					 if($l_cv<.7){
						 if($i <= ($dus+19)){
						    if(($i-$dus+1)>$l_kill){
						       $l_kill=$i-$dus+1; 
						    }
						 }
						 if($i >= ($due-19)){
						    if(($due-$i+1)>$r_kill){
						       $r_kill=$due-$i+1;  
						    }
						 } 
					 }
				 }else{
					 if($string->{$i}{$add}>2){
					 }else{
						 if($i <= ($dus+19)){
						    if(($i-$dus+1)>$l_kill){
						       $l_kill=$i-$dus+1; 
						    }
						 }
						 if($i >= ($due-19)){
						    if(($due-$i+1)>$r_kill){
						       $r_kill=$due-$i+1;  
						    }
						 }
					 }
				 }
			 }else{
				 if(($i-$dus+1)>$l_kill){
				    $l_kill=$i-$dus+1; 
				 }	
				 #print "\ndefined nist  i $i node $node s $dus e $due l_kill $l_kill - $contig\n"; 				 
			 }
		  }	  		   
          if($L>($contig_size_cutoff+$l_kill+$r_kill)){ 
			  for (my $i=$dus; $i <= ($dus+$l_kill-1) ; $i++){
				  foreach my $pass (keys %{$whoshere->{$i}}){  
					 delete $track->{$pass};
				  }	
			  }
			  for (my $i=($due-$r_kill+1); $i <= $due ; $i++){
				  foreach my $pass (keys %{$whoshere->{$i}}){  
					 delete $track->{$pass};
				  }	
			  }  
			  $contig=substr($contig,$l_kill); $contig=substr($contig,0,(length($contig)-$r_kill)); 
			  $new_cov-=$r_kill; $new_cov-=$l_kill;
		  }else{
		      $l_kill=0; $r_kill=0;
		  }
		  $L=length($contig);
          if($L>($contig_size_cutoff)){
			  $new_cov/=$L;
			  printf TIG ">contig%i|size%i|cov%.2f\n%s\n", ($node,$L,$new_cov,$contig);   
			  $tig_info->{$node}{'cov'}=.01* int(100*$new_cov);
			  $tig_info->{$node}{'length'}=$L;	  
			  $move+=$l_kill;
			  ($track_all,$black_pass)= &trackReads($track,$track_all,$move,$black_pass); 
			  if( $node % 500 == 0 ){
				  my $date =`date`; chomp($date); print "contig $node $date\n";
			  }
		  }else{
		      #print "too short node $node L $L\n"; 
		      $node--;  
		  }
          undef $string; undef $whoshere, undef $bad_pass; undef $track;        
       }
       $node++;  
   }elsif($line=~/^\>/){
       my @title = split(/,/,$line);
       my $start=$title[1];
       my $read=<TEMP>; chomp $read;  
       if($read =~/^([ACGT]*)$/){
           ($string,$whoshere)= updatestring($string, $read, $start, $whoshere);
           ($track, $bad_pass)=updatelocaltrack($track, $read, $start, $node, $bad_pass);        
       }else{
           
       }   
   }
}
#----------------------------
if($node>0){
  my @position= sort {$a <=> $b} (keys %{$string});  
  my $L=scalar(@position); my ($l_kill,$r_kill)=(0,0);  
  my $contig;
  my $dus=shift(@position); my $due=pop(@position); my $move=$dus-1; my $new_cov;
  for (my $i=$dus; $i <= $due ; $i++){
	 if(defined $string->{$i}){
		 my $add;
		 foreach my $col (sort { $string->{$i}{$b} <=> $string->{$i}{$a} } keys %{$string->{$i}}){     
			 $contig.=$col;
			 $add=$col;
			 last;
		 }  
		 $new_cov+=$string->{$i}{$add}; 
		 if(scalar(keys %{$string->{$i}})>1){
			 my $coverage=0; 
			 foreach my $j (@AG){
				 if ( defined $string->{$i}{$j} ) {
					 $coverage+=$string->{$i}{$j};
				 }
			 }	
			 my $l_cv=$string->{$i}{$add}/$coverage;
			 if($l_cv>.9){
				 foreach my $pass (keys %{$whoshere->{$i}}){
					  if($whoshere->{$i}{$pass} eq $add){
					  }else{
						  delete $track->{$pass};
					  }
				 }
			 }else{
				 foreach my $pass (keys %{$whoshere->{$i}}){
					 delete $track->{$pass};
				 }					 		 
			 }
			 if($l_cv<.7){
				 if($i <= ($dus+19)){
					if(($i-$dus+1)>$l_kill){
					   $l_kill=$i-$dus+1; 
					}
				 }
				 if($i >= ($due-19)){
					if(($due-$i+1)>$r_kill){
					   $r_kill=$due-$i+1;  
					}
				 } 
			 }
		 }else{
			 if($string->{$i}{$add}>2){
			 }else{
				 if($i <= ($dus+19)){
					if(($i-$dus+1)>$l_kill){
					   $l_kill=$i-$dus+1; 
					}
				 }
				 if($i >= ($due-19)){
					if(($due-$i+1)>$r_kill){
					   $r_kill=$due-$i+1;  
					}
				 }
			 }
		 }
	 }else{
		 #print "\ndefined nist  i $i node $node\n"; 
	 }
  }	  		  
  foreach my $rd (keys %{$bad_pass}){
	   if (defined $track->{$rd}){
		   print "eee e e eeee\n";
	   }
  }		  
  if($L>($contig_size_cutoff+$l_kill+$r_kill)){ 
	  for (my $i=$dus; $i <= ($dus+$l_kill-1) ; $i++){
		  foreach my $pass (keys %{$whoshere->{$i}}){  
			 delete $track->{$pass};
		  }	
	  }
	  for (my $i=($due-$r_kill+1); $i <= $due ; $i++){
		  foreach my $pass (keys %{$whoshere->{$i}}){  
			 delete $track->{$pass};
		  }	
	  }  
	  $contig=substr($contig,$l_kill); $contig=substr($contig,0,(length($contig)-$r_kill)); 
	  $new_cov-=$r_kill; $new_cov-=$l_kill;
  }else{
	  $l_kill=0; $r_kill=0;
  }
  $L=length($contig);
  if($L>($contig_size_cutoff)){
	  $new_cov/=$L;
	  printf TIG ">contig%i|size%i|cov%.2f\n%s\n", ($node,$L,$new_cov,$contig); 
	  $tig_info->{$node}{'cov'}=.01* int(100*$new_cov);
	  $tig_info->{$node}{'length'}=$L;
	  $move+=$l_kill;
	  ($track_all,$black_pass)= &trackReads($track,$track_all,$move,$black_pass); 
  }else{
	  #print "too short node $node L $L\n"; 
	  $node--;  
  }
  undef $string; undef $whoshere, undef $bad_pass; undef $track;        
}
close TEMP;

unlink("$address"."/s_tmp/temp") or die "Cannot delete temp file\n";
rmdir "$address"."/s_tmp";

$track_all = &Filtertrack($track_all, $black_pass); print "\n";
#--------------------------------  

my @aray_ins = Empiricalsize($pairead_file, $track_all, $forbiden_link, $tig_info, $node);  

($Ori_Dis, $selfpair, $tig_info) = OrientationDistance($pairead_file, $track_all, $forbiden_link, $tig_info, $node, @aray_ins);  

store \%{$Ori_Dis}, "$address"."/orientdistinfo". $base_name; 
undef $Ori_Dis; 
	
#--------------------------------  
#print "du_badpas $du_badpas du_badpas2 $du_badpas2 du_badpas3 $du_badpas3 du_badpas4 $du_badpas4 du_badpas5 $du_badpas5 du_badpas6 $du_badpas6 du_badpas7 $du_badpas7 du_badpas8 $du_badpas8 du_badpas9 $du_badpas9\n";
#print "k1 $kharab1 k2 $kharab2 k3 $kharab3 k4 $kharab4 selfpair $selfpair\n";

open (SF, ">$address"."/badselfpair_c$copy_li"."_e$ign_empi"."$base_name".".txt") or die "$!";   
foreach my $a (keys %$tig_info){
	if($tig_info->{$a}{'in_consis'}>20){
	   print SF  "contig $a number of badpair $tig_info->{$a}{'in_consis'}\n";
	}
}
close SF;

$tig_info->{1}{'seq_l_u'}=$s_l_u;
$tig_info->{1}{'seq_l_d'}=$s_l_d;

my @lengths; my $sum;
foreach my $a (keys %$tig_info){
     push @lengths, $tig_info->{$a}{'length'} ;
     $sum+=$tig_info->{$a}{'length'} ;
}	
@lengths=sort {$b <=> $a} @lengths;  

my $larges=0;my $N50;

SBASSE: for (my $i=0; $i < scalar(@lengths); $i++){        
      $larges+=$lengths[$i];
      if($larges>($sum/2)){
            $N50=$lengths[$i];
            last SBASSE;
      }
}

store \%{$tig_info}, "$address"."/div/tig_info". $base_name;  

open (CDIS, ">$address"."/div/coverage_distribution.txt");

foreach my $tig ( keys %{$tig_info}){  
	  print CDIS "$tig_info->{$tig}{'cov'} ";
}

close CDIS;

#-------------------------------------------------
my $c_n;
open(IN,$afg_file) or die "Cannot open $afg_file $!\n";
while(<IN>){
    chomp();
    if( $_ =~ /^\{RED/){
       my $line=<IN>; chomp($line);
       if ( $line =~ /^iid/ ){
       
	   }else{

	   }
	   $line=<IN>; $line=<IN>;$line=<IN>; chomp($line);
	   if( $line =~ /^seq/){
	   		 #$line=<IN>; chomp($line);
	   }
	   if( $line =~ /^([ACGT]*)$/){
	       my $read=$line;    
	       $line=<IN>; chomp($line);
	       while( $line =~ /^([ACGT]*)$/){
	             $read.=$line;
	             $line=<IN>; chomp($line);
	       }

		   if ( defined $c_n->{$read} ){
		        $c_n->{$read}++;
		   }elsif ( defined $c_n->{reverseComplement($read)} ){
		        $c_n->{reverseComplement($read)}++;
		   }else{
		        $c_n->{$read}++;
		   }	        
	   }else{
	   }
    }
}
close IN; 
open (CN, ">$address"."/div/copynumber.txt" ) or die "$!";
foreach my $r ( keys %{$c_n} ){
     print CN "$c_n->{$r} ";
}
close CN; undef $c_n;

$date = `date`; chomp($date); print "\nFinished contig building $date     N50 $N50     sum $sum\n\n";   

exit;

#------------------------------------
sub Filtertrack{
   my ($track_all, $black_pass) = @_;  
   
   foreach my $rd (keys %{$track_all}){
       my $count=$track_all->{$rd}{'count'};
       
       my $rv_rd=reverseComplement($rd);
       if (defined $track_all->{$rv_rd}){
            if(($track_all->{$rv_rd}{'tig'}!=$track_all->{$rd}{'tig'}) || ($track_all->{$rv_rd}{'start'}!=$track_all->{$rd}{'end'}) || ($track_all->{$rv_rd}{'end'}!=$track_all->{$rd}{'start'})){
                 $black_pass->{$rd}++;
                 $black_pass->{$rv_rd}++;
                 $du_badpas6++;  #print "tg1 $track_all->{$rv_rd}{'tig'} tg2 $track_all->{$rd}{'tig'} s1 $track_all->{$rv_rd}{'start'} e2 $track_all->{$rd}{'end'} e1 $track_all->{$rv_rd}{'end'} s2 $track_all->{$rd}{'start'}\n";
            }else{
                 $count+=$track_all->{$rv_rd}{'count'};
                 $du_badpas7++;
            }
       }
       if($count < $copy_li){
       
       }else{
            $black_pass->{$rd}++;
            $du_badpas4++;
       }
   }
   foreach my $rd (keys %{$black_pass}){
       delete $track_all->{$rd};
       $du_badpas5++;
   }
   return $track_all;
}
#------------------------------------
sub trackReads{
   my ($track, $track_all,$move,$black_pass) = @_;  
   foreach my $rd (keys %{$track}){
      if($track->{$rd}{'count'}<$copy_li){ 
		  if(! defined $track_all->{$rd}{'tig'}){
			 $track_all->{$rd}{'tig'} = $track->{$rd}{'tig'};
			 $track_all->{$rd}{'start'} = $track->{$rd}{'start'}-$move;
			 $track_all->{$rd}{'end'} = $track->{$rd}{'end'}-$move;
			 $track_all->{$rd}{'count'}= $track->{$rd}{'count'};
			 delete $track->{$rd};
		  }else{
			 if(($track->{$rd}{'tig'}!=$track_all->{$rd}{'tig'}) || ($track->{$rd}{'start'}!=$track_all->{$rd}{'start'}) || ($track->{$rd}{'end'}!=$track_all->{$rd}{'end'})){
				 $du_badpas3++;
				 $black_pass->{$rd}++;
			 }else{
			     $track_all->{$rd}{'count'}++;
			 }
		  }
	  }else{
	      $du_badpas2++;
	      $black_pass->{$rd}++;
	  }
   }
   return $track_all, $black_pass;
}
#--------------------------------     
sub updatelocaltrack{
   my ($track, $pass, $shoroo, $tig_num, $bad_pass) = @_;
   if ($shoroo>=0){
	  $shoroo++;
	  if (!defined $bad_pass->{$pass}){
		  if(!defined $track->{$pass}){
			  $track->{$pass}{'tig'}=$tig_num;
			  $track->{$pass}{'start'}=$shoroo;
			  $track->{$pass}{'end'}=$shoroo+length($pass)-1; 
			  $track->{$pass}{'count'}++;
		  }else{
			  if(($track->{$pass}{'tig'}!=$tig_num) || ($track->{$pass}{'start'}!=$shoroo) || ($track->{$pass}{'end'}!=($shoroo+length($pass)-1))){
				   $du_badpas++;
				   delete $track->{$pass};
				   $bad_pass->{$pass}++;
			  }else{
				   $track->{$pass}{'count'}++;
			  }
		  }
      }else{
		  $bad_pass->{$pass}++;
      }
   }else{
	  $shoroo*=-1;
	  $shoroo++;
	  if (!defined $bad_pass->{$pass}){
		  if(!defined $track->{$pass}){
			  $track->{$pass}{'tig'}=$tig_num;
			  $track->{$pass}{'start'}=$shoroo;
			  $track->{$pass}{'end'}=$shoroo-length($pass)+1;
			  $track->{$pass}{'count'}++;
		  }else{
		      if(($track->{$pass}{'tig'}!=$tig_num) || ($track->{$pass}{'start'}!=$shoroo) || ($track->{$pass}{'end'}!=($shoroo-length($pass)+1))){
		           $du_badpas++;
		           delete $track->{$pass};
		           $bad_pass->{$pass}++;
			  }else{
				   $track->{$pass}{'count'}++;
			  }
		  }
      }else{
		  $bad_pass->{$pass}++;
      }            
   }  
   return $track, $bad_pass;
}   
#--------------------------------
sub updatestring{
   my ($reshte, $short_read, $shoroo, $kinjast) = @_;
   if ($shoroo>=0){
      my @slice_read=split(//,$short_read);
	  $shoroo++;
	  for (my $i=0; $i < scalar(@slice_read) ; $i++){
	     $reshte->{$shoroo+$i}{$slice_read[$i]}++; 
	     $kinjast->{$shoroo+$i}{$short_read}=$slice_read[$i];
	  }
   }else{
      my @slice_read=split(//,Complement($short_read));
	  $shoroo*=-1;  
	  $shoroo++;
	  for (my $i=0; $i < scalar(@slice_read) ; $i++){
		 $reshte->{$shoroo-$i}{$slice_read[$i]}++; 
		 $kinjast->{$shoroo-$i}{$short_read}=$slice_read[$i];
	  }	 
   }  
   return $reshte, $kinjast;
}  
#--------------------------------
sub  OrientationDistance{    
   my ($file, $track, $forbiden, $info, $mtig, @aray_in) = @_;
   my $ORI_DIS; my $num_self; my $insert_size; my $dev_1; my $dev_2;
 
   my $mean=0; my $N=0;
   foreach my $tig (keys %{$info}){  
        $mean+=$info->{$tig}{'cov'};  $N++; $info->{$tig}{'in_consis'}=0;
   }

#    $mean=int($mean/$N); my $cut=2*$mean; print "for ori-dis nmtig $mtig mean_cov $mean cut $cut\n";
#    for(my $tig=1;$tig<$mtig;$tig++){
#        if(($info->{$tig}{'length'}>(2*$insert_size)) && ($info->{$tig}{'cov'}<$cut) ){
#            my $tot_window=int($info->{$tig}{'length'}/$seq_l);
#            $info->{$tig}{'tot_w'}=$tot_window; 
#            for(my $i=1;$i<=$info->{$tig}{'tot_w'};$i++){
#                $info->{$tig}{'pilar'}{$i}=0; 
#            }
#        }
#    }
   open(IN,$file) || die "Can't open $file -- fatal\n";
   AKHAREMATE: while( my $line=<IN> ){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
           if ($ign_empi==1){
           		$insert_size = $2;
           }else{
		        $insert_size = shift(@aray_in);   
           }
		   print "Associated library: $1\n";
           print "Suggested insert_size: $2         We will use: $insert_size\n\n";
          if($insert_size>5000){
               $dev_1=1.1; $dev_2=.2;
          }elsif($insert_size>1000){
               $dev_1=1.3; $dev_2=.5;
          }else{
               $dev_1=1.6; $dev_2=1;
          }          
          #print "insert_size $insert_size dev_1 $dev_1 dev_2 $dev_2\n";          
      }elsif($line =~ /^Fragment$/){
          last AKHAREMATE;     
      }elsif($line =~ /^([^\>]*)$/i){  
         my $sdna = $1;  
         if($sdna =~/^([ACGT]*)\:([ACGT]*)$/i){
            my $read_a=$1; my $read_b=$2; 
            
            if((defined $track->{$read_a}) && (defined $track->{$read_b})){### both pairs assembled				     
				 if((!defined $track->{$read_a}{'tig'}) || (!defined $track->{$read_b}{'tig'})){
					print "unexpected problem\n"; exit;
				 }
				 my $tig_a = $track->{$read_a}{'tig'};
				 my $tig_b = $track->{$read_b}{'tig'};    
				 my $A_length = $info->{$tig_a}{'length'};
				 my $A_start = $track->{$read_a}{'start'};
				 my $A_end = $track->{$read_a}{'end'};
				 my $B_length = $info->{$tig_b}{'length'};
				 my $B_start = $track->{$read_b}{'start'} ;
				 my $B_end = $track->{$read_b}{'end'};
				 
				 #if(($A_length>100) && ($B_length>100)){		  
					 if($tig_a != $tig_b){####paired reads located on <> contigs
						if(! defined $forbiden->{$tig_a}{$tig_b}){
							if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
								  if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> -> 
									 my $d=$insert_size+$A_start-$B_start;
					  
									 my $gap=$d-$info->{$tig_a}{'length'};
									 if(abs($gap)<($dev_1*$insert_size)){
											 if ($tig_a<$tig_b){
												$ORI_DIS->{$tig_a}{$tig_b}{'P'}++;
												$ORI_DIS->{$tig_a}{$tig_b}{'PD'}+=$d;											   
											 }else{    
												$ORI_DIS->{$tig_b}{$tig_a}{'P'}++;
												$ORI_DIS->{$tig_b}{$tig_a}{'PD'}-=$d;				                               
											 }							 
									  }else{
										 $kharab1++;
									  }
								  }else{# -> <-
									 my $d=$insert_size+$A_start+$B_start;
															 
									 my $gap=$d-$info->{$tig_a}{'length'}-$info->{$tig_b}{'length'};
									 if(abs($gap)<($dev_1*$insert_size)){
											 if ($tig_a<$tig_b){
												$ORI_DIS->{$tig_a}{$tig_b}{'N'}++;
												$ORI_DIS->{$tig_a}{$tig_b}{'ND'}+=$d;
											 }else{
												$ORI_DIS->{$tig_b}{$tig_a}{'N'}++;
												$ORI_DIS->{$tig_b}{$tig_a}{'ND'}+=$d;												
											 }
									 }else{                              
										 $kharab2++;
									 }
								  }
							}else{
								  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  <-  <-
									 my $d=-($insert_size-$A_start+$B_start);	
					  
									 my $gap=-$d-$info->{$tig_b}{'length'};
									 if(abs($gap)<($dev_1*$insert_size)){
											 if ($tig_a<$tig_b){
												$ORI_DIS->{$tig_a}{$tig_b}{'P'}++;
												$ORI_DIS->{$tig_a}{$tig_b}{'PD'}+=$d;
											 }else{
												$ORI_DIS->{$tig_b}{$tig_a}{'P'}++;
												$ORI_DIS->{$tig_b}{$tig_a}{'PD'}-=$d;												
											 }
									 }else{							 
										 $kharab3++;
									 }
								  }else{    #   <- ->  
									 my $d=-($insert_size-$A_start-$B_start);
									 
									 my $gap=-$d;
									 if(abs($gap)<($dev_1*$insert_size)){
											 if ($tig_a<$tig_b){
												$ORI_DIS->{$tig_a}{$tig_b}{'N'}++;
												$ORI_DIS->{$tig_a}{$tig_b}{'ND'}+=$d;					
											 }else{
												$ORI_DIS->{$tig_b}{$tig_a}{'N'}++;
												$ORI_DIS->{$tig_b}{$tig_a}{'ND'}+=$d;											
											 }
									 }else{							 
										 $kharab4++
									 }									 
								  }
							}
						}
					 }else{###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
# 						  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
# 								if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> -> 
# 								      my $du= $B_start-$A_start;
# 									  my $d=$B_start-$A_start-$insert_size;
# 									  if (abs($d)<($dev_2*$insert_size)){
# 										   $num_self++;
# 										   if(defined $info->{$tig_a}{'tot_w'}){ 
# 											   my $w_s; my $w_e;
# 											   my $dus=1+int($track->{$read_a}{'start'}/$seq_l);
# 											   if($dus>$info->{$tig_a}{'tot_w'}){  
# 												  $w_s=$info->{$tig_a}{'tot_w'}; 
# 											   }else{
# 												  $w_s=$dus;
# 											   }
# 											   undef $dus; my $due=1+int($track->{$read_b}{'end'}/$seq_l);
# 											   if($due>$info->{$tig_a}{'tot_w'}){  
# 												   $w_e=$info->{$tig_a}{'tot_w'};
# 											   }else{
# 												  $w_e=$due;
# 											   } 
# 											   for(my $i=$w_s;$i<=$w_e;$i++){
# 													$info->{$tig_a}{'pilar'}{$i}++; 
# 											   }
# 										   }
# 									  }else{
# 										  $info->{$tig_a}{'in_consis'}++; 
# 									  }
# 								}else{     
# 								   $info->{$tig_a}{'in_consis'}++;
# 								}
# 						  }else{
# 							  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  <-  <-
# 									  my $d=$A_start-$B_start-$insert_size;
# 									  if (abs($d)<($dev_2*$insert_size)){
# 										   $num_self++;
# 										   if(defined $info->{$tig_a}{'tot_w'}){ 
# 											   my $w_s; my $w_e;
# 											   my $dus=1+int($track->{$read_b}{'end'}/$seq_l);
# 											   if($dus>$info->{$tig_a}{'tot_w'}){  
# 												  $w_s=$info->{$tig_a}{'tot_w'}; 
# 											   }else{
# 												  $w_s=$dus;
# 											   }
# 											   undef $dus; my $due=1+int($track->{$read_a}{'start'}/$seq_l);
# 											   if($due>$info->{$tig_a}{'tot_w'}){  
# 												  $w_e=$info->{$tig_a}{'tot_w'};
# 											   }else{
# 												  $w_e=$due;
# 											   } 
# 											   for(my $i=$w_s;$i<=$w_e;$i++){
# 													$info->{$tig_a}{'pilar'}{$i}++; 
# 											   }
# 										   }	   
# 									  }else{
# 										  $info->{$tig_a}{'in_consis'}++; 
# 									  }
# 							  
# 							  }else{     
# 								   $info->{$tig_a}{'in_consis'}++;
# 							  }
# 							  
# 						  }	  
					 } 
				 #} 
		    }     
         }
      }
   }
   close IN;  
   return $ORI_DIS, $num_self, $info;
}
#----------------------------------------------
sub  Empiricalsize{    
   my ($file, $track, $forbiden, $info, $mtig) = @_;
   
   my @aray_in; my @sizes; my $num_self; my $insert_size; my $dev_1; my $dev_2;  my $mm=0; my $min_n=200; my $assoc;
 
   my $mean=0; my $N=0;
   foreach my $tig (keys %{$info}){  
        $mean+=$info->{$tig}{'cov'};  $N++; $info->{$tig}{'in_consis'}=0;
   }  

   open(IN,$file) || die "Can't open $file -- fatal\n";
   AKHAREMATE: while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
          if ($mm==1) {          
		      print "Associated library: $assoc\n";
			  if ( scalar(@sizes) > 0){
					  @sizes= sort @sizes;   
					  my $L=scalar(@sizes);    

					 my $sum=0;
					 foreach my $t (@sizes){
						   $sum+=$t; 
					  }
					  my $new_i=int ($sum/$L)+1;
					  if ( $L > $min_n ){
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
							 push @aray_in, $new_i;     
					  }else{
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
							 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
							 push @aray_in, $insert_size ;    
					  }
					  
			  }else{
    	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
					push @aray_in, $insert_size ; 
              }
          }
          $mm=1;
          $insert_size=$2; $assoc=$1;

          my $assoc_trim=$1;
          
          if($assoc_trim =~ /(.*)\/([^\/]*)$/){
          	 $assoc_trim=$2;
          
          }else{
          
          } 
          
          open (DDIS, ">$address"."/div/insertsize_distribution_$assoc_trim"."_$insert_size".".txt");
          
          undef @sizes;
          if($insert_size>5000){
               $dev_1=1.1; $dev_2=2;
          }elsif($insert_size>1000){
               $dev_1=1.3; $dev_2=2.5;
          }elsif($insert_size>500){
               $dev_1=1.6; $dev_2=3.25;
          }elsif($insert_size>250){
               $dev_1=1.6; $dev_2=3.75;
          }else{
               $dev_1=1.6; $dev_2=4;
          }           
          #print "insert_size $insert_size dev_1 $dev_1 dev_2 $dev_2\n";          
      }elsif($line =~ /^Fragment$/){
          if ($mm==1) {          
		      print "Associated library: $assoc\n";
			  if ( scalar(@sizes) > 0){
					  @sizes= sort @sizes;   
					  my $L=scalar(@sizes);    

					 my $sum=0;
					 foreach my $t (@sizes){
						   $sum+=$t; 
					  }
					  my $new_i=int ($sum/$L)+1;
					  if ( $L > $min_n ){
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
							 push @aray_in, $new_i;     
					  }else{
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
							 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
							 push @aray_in, $insert_size ;    
					  }
					  
			  }else{
    	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
					push @aray_in, $insert_size ; 
              }
          }
          $mm=0;          
          last AKHAREMATE;     
      }elsif($line =~ /^([^\>]*)$/i){  
         my $sdna = $1;  
         if($sdna =~/^([ACGT]*)\:([ACGT]*)$/i){
            my $read_a=$1; my $read_b=$2; 
            
            if((defined $track->{$read_a}) && (defined $track->{$read_b})){### both pairs assembled				     
				 if((!defined $track->{$read_a}{'tig'}) || (!defined $track->{$read_b}{'tig'})){
					print "unexpected problem\n"; exit;
				 }
				 my $tig_a = $track->{$read_a}{'tig'};
				 my $tig_b = $track->{$read_b}{'tig'};    
				 my $A_length = $info->{$tig_a}{'length'};
				 my $A_start = $track->{$read_a}{'start'};
				 my $A_end = $track->{$read_a}{'end'};
				 my $B_length = $info->{$tig_b}{'length'};
				 my $B_start = $track->{$read_b}{'start'} ;
				 my $B_end = $track->{$read_b}{'end'};
				 #if(($A_length>100) && ($B_length>100)){		  
					 if($tig_a != $tig_b){####paired reads located on <> contigs
					 }else{###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
					     if( $A_length > (2*$insert_size) ){				  
							  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
									if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'} ){#  -> <- 
										  
										  my $imp= $B_start-$A_start;  print DDIS "$imp ";
										  my $d=$B_start-$A_start-$insert_size;
										  
										  if ( (abs($d)<($dev_2*$insert_size)) && ($imp>0)){
											   push @sizes, $imp; 
										  }else{
											   
										  }
										  
										  
									}else{     
									   
									}
							  
							  }else{
								  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'}){#  ->  <-
									   
										  my $imp= $A_start-$B_start; print DDIS "$imp ";
										  my $d=$A_start-$B_start-$insert_size;
										  
										  if ( (abs($d)<($dev_2*$insert_size)) && ($imp>0)){
											   push @sizes, $imp;  
										  }else{
											   
										  }
								  
								  }else{     
									   
								  }
								  
							  }
						 }						  
					 } 
				 #} 
		    }     
         }
      }
   }
   if ($mm==1) {          
		  print "Associated library: $assoc\n";
		  if ( scalar(@sizes) > 0){
				  @sizes= sort @sizes;   
				  my $L=scalar(@sizes);    

				 my $sum=0;
				 foreach my $t (@sizes){
					   $sum+=$t; 
				  }
				  my $new_i=int ($sum/$L)+1;
				  if ( $L > $min_n ){
						 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
						 push @aray_in, $new_i;     
				  }else{
						 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
						 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
						 push @aray_in, $insert_size ;    
				  }
				  
		  }else{
   	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
   	  	   	    push @aray_in, $insert_size ; 
          }
   }   
   close IN; close DDIS; print "\n";
   return @aray_in;
}
#----------------------------------------------
sub Complement{
   $_ = shift;
   $_ = uc();
   tr/ATGC/TACG/;
   return $_;
}
#-----------------
sub reverseComplement{
   $_ = shift;
   $_ = uc();
   tr/ATGC/TACG/;
   return (reverse());
}
#-----------------------