#!/usr/bin/perl

#AUTHOR
#  Adel Dayarian

#The code is available freely, under the GNU Public License, at http://www.physics.rutgers.edu/~anirvans/SOPRA/

use Storable;
use strict;
use Data::Dumper;
use Getopt::Long;

my $opt_f;
my $opt_z;
my $opt_c;
my $opt_e;
my $opt_a;

my $address="";

&GetOptions( "f=s",  => \$opt_f,
     	     "z=i",  => \$opt_z,
     	     "c=i",  => \$opt_c,
     	     "e=i",  => \$opt_e,
     	     "a=s", => \$opt_a,
           ) or die "Unknown option\n";
           
           
use vars qw($opt_f);
use vars qw($opt_z);
use vars qw($opt_c);
use vars qw($opt_a);
use vars qw($opt_e);

&GetOptions( "f=s", 
             "z=i", 
     	     "c=i",  
     	     "e=i",  
     	     "a=s",  
           ) ; 
           
my ($pairead_file,$contig_size_cutoff,$s_l_u,$s_l_d,$copy_li,$ign_empi)=('sread_in.fasta',0,0,0,5,0);
#-------------------------------------------------
if( (! $opt_f) || (! $opt_a) ){ 
   print "Usage: $0\n";
   print "-f  Output of 'solid_denovo_postprocessor.pl' (run Velvet in fragment mode)\n";
   print "-a  The directory that you used for output of the formating script (format_base.pl)\n\n";   

   print "-z Minimum contig size (default 1.5*shortest_read_length)\n";
   print "-e  If set equal to 1, the empirical value for the insert size will not be used\n";   
   print "-c  If the number of times a read and its reverse complement appear in the library is equal to or more than this value, the pairing information from that read will be disregarded  (default -c 5)\n\n";
   
   print "example: $0 -f output_of_solid_denovo_postprocessor.pl -a mydir_sout\n\n";
   exit;
}
my $date = `date`; chomp($date); print "\nStart $date\n\n";
my $postprocess_file = $opt_f;
$contig_size_cutoff = $opt_z if ($opt_z);
$copy_li=$opt_c if($opt_c);
if( $opt_e == 1 ){
      $ign_empi=1;
}

$address= $opt_a;

if( $address =~ /^(.*)\/$/ ){
     $address=$1; 
}
if (-d $address) {
	#print "There is a directory named $address\n";
}else {
	print "There is no such directory $address\n"; exit;
}

$pairead_file=$address."/$pairead_file";

mkdir("$address"."/div", 0777) || print $!;
#-------------------------------------------------
my $pid_num = getpgrp(0);  
my $init_message = "\nRunning: $0\n-f $postprocess_file\n-c $copy_li\n-e $ign_empi\n-a $address\n\n";
print $init_message;
print "pid $pid_num\n\n";  
#-------------------------------------------------
open(IN,$postprocess_file) || die "Can't open $postprocess_file -- fatal\n";
while(my $line=<IN>){
   if($line=~/^=NODE.*$/){
   
   }elsif($line=~/^\>/){    
  
       my $read=<IN>; chomp $read;
       
       if($read =~/^[ACGT]([0-3]*)$/){
		   if( $s_l_d == 0 ){
			   $s_l_d=length($read)-1;
		   }elsif( length($read) < $s_l_d ){
			   $s_l_d=length($read)-1;
		   }
		   if( length($read) > $s_l_u ){
			   $s_l_u=length($read)-1;
		   }            
       }else{
           print "format?\n";
       }
   }
}
close IN; 

if($contig_size_cutoff==0){
	$contig_size_cutoff=int(1.5*$s_l_d); 
}
print "max read length $s_l_u bp      min read length $s_l_d bp       contig_size_cutoff $contig_size_cutoff\n\n";

my $seq_l=$s_l_u;
#-------------------------------------------------
my ($string, $track_all, $track, $whoshere, $bad_pass, $black_pass, $tig_info, $forbiden_link, $Ori_Dis, $selfpair); my $node=0;
my ($du_badpas, $du_badpas2, $du_badpas3, $du_badpas4, $du_badpas5, $du_badpas6, $du_badpas7, $kharab1, $kharab2, $kharab3, $kharab4);
  
open(IN,$postprocess_file) || die "Can't open $postprocess_file -- fatal\n";

my $base_name .= "_pid" . $pid_num;

$date = `date`; chomp($date); 
print "Contig reconstruction started    $date    pid $pid_num\n\n";  

open (TIG, ">$address"."/contigs_colorspace". $base_name .".fasta" ) or die "$!";

my @AG = (0,1,2,3);

while(my $line=<IN>){
   if($line=~/^=NODE.*$/){
       if($node>0){
          my @position= sort {$a <=> $b} (keys %{$string});  
          my $L=scalar(@position); my ($l_kill,$r_kill)=(0,0); my $l_cv; my $new_cov;
          my $contig;
          my $dus=shift(@position); my $due=pop(@position); my $move=$dus-1;          
		  for (my $i=$dus; $i <= $due ; $i++){
			 if(defined $string->{$i}){
				 my $add;
				 foreach my $col (sort { $string->{$i}{$b} <=> $string->{$i}{$a} } keys %{$string->{$i}}){     
					 $contig.=$col;
					 $add=$col;
					 last;
				 }  
				 $new_cov+=$string->{$i}{$add}; 
				 if(scalar(keys %{$string->{$i}})>1){
					 my $coverage=0; 
					 foreach my $j (@AG){
						 if ( defined $string->{$i}{$j} ) {
							 $coverage+=$string->{$i}{$j};
						 }
					 }						 
					 $l_cv->{$i}=$string->{$i}{$add}/$coverage;
					 if($l_cv->{$i}>.9){
						 foreach my $pass (keys %{$whoshere->{$i}}){
							  if($whoshere->{$i}{$pass} eq $add){
							  }else{
								  $bad_pass->{$pass}++;
							  }
						 }
					 }else{
						 foreach my $pass (keys %{$whoshere->{$i}}){
							 $bad_pass->{$pass}++;
						 }					 		 
					 }
					 if($l_cv->{$i}<.7){
						 if($i <= ($dus+19)){
						    if(($i-$dus+1)>$l_kill){
						       $l_kill=$i-$dus+1; 
						    }
						 }
						 if($i >= ($due-19)){
						    if(($due-$i+1)>$r_kill){
						       $r_kill=$due-$i+1;  
						    }
						 } 
					 }
				 }else{
					 $l_cv->{$i}=1;
					 if($string->{$i}{$add}>2){
					 }else{
						 if($i <= ($dus+19)){
						    if(($i-$dus+1)>$l_kill){
						       $l_kill=$i-$dus+1; 
						    }
						 }
						 if($i >= ($due-19)){
						    if(($due-$i+1)>$r_kill){
						       $r_kill=$due-$i+1;  
						    }
						 }
					 }
				 }
			 }else{
				#print "\ndefined nist  i $i node $node dus $dus due $due\n"; 
			 }
		  }	  		  
          if($L>($contig_size_cutoff+$l_kill+$r_kill)){ 
			  for (my $i=$dus; $i <= ($dus+$l_kill-1) ; $i++){
				  foreach my $pass (keys %{$whoshere->{$i}}){  
					 delete $track->{$pass};
				  }	
			  }
			  for (my $i=($due-$r_kill+1); $i <= $due ; $i++){
				  foreach my $pass (keys %{$whoshere->{$i}}){  
					 delete $track->{$pass};
				  }	
			  }  
			  $contig=substr($contig,$l_kill); $contig=substr($contig,0,(length($contig)-$r_kill)); 
			  $new_cov-=$r_kill; $new_cov-=$l_kill;
		  }else{
		      $l_kill=0; $r_kill=0;
		  }
		  $L=length($contig);
          if($L>($contig_size_cutoff)){
			  $new_cov/=$L;
			  printf TIG ">contig%i|size%i|cov%.2f\n%s\n", ($node,$L,$new_cov,$contig); 
			  $tig_info->{$node}{'cov'}=.01* int(100*$new_cov);
			  $tig_info->{$node}{'length'}=$L;
			  $move+=$l_kill;
			  ($track_all,$black_pass)= &trackReads($track,$track_all,$bad_pass,$move,$black_pass); 
			  foreach my $i (keys %{$l_cv}){
			      $tig_info->{$node}{'Cvalue'}{$i-$move}=$l_cv->{$i};
			  }
			  if ( $node % 500 == 0 ){
			        my $date = `date`; chomp($date); print "contig $node $date\n";
			  }
		  }else{
		      #print "too short node $node L $L\n"; 
		      $node--;  
		  }
          undef $string; undef $whoshere, undef $bad_pass; undef $track;        
       }
       $node++;       
   }elsif($line=~/^\>/){
       chomp $line;
       my @title = split(/,/,$line);
       my $start=$title[1];
      
       my $pread=<IN>; 
       chomp $pread;
       if($pread =~/^[ACGT]([0-3]*)$/){
           my $read=substr($1,1);
           ($string,$whoshere)= updatestring($string, $read, $start, $whoshere);
           ($track, $bad_pass)=updatelocaltrack($track, $read, $start, $node, $bad_pass);           
       }else{
          
       }
       
   }
}
close IN;
 #--------------------------------
if($node>0){
  my @position= sort {$a <=> $b} (keys %{$string});  
  my $L=scalar(@position); my ($l_kill,$r_kill)=(0,0); my $l_cv; my $new_cov;
  my $contig;
  my $dus=shift(@position); my $due=pop(@position); my $move=$dus-1;          
  for (my $i=$dus; $i <= $due ; $i++){
	 if(defined $string->{$i}){
		 my $add;
		 foreach my $col (sort { $string->{$i}{$b} <=> $string->{$i}{$a} } keys %{$string->{$i}}){     
			 $contig.=$col;
			 $add=$col;
			 last;
		 }  
		 $new_cov+=$string->{$i}{$add}; 
		 if(scalar(keys %{$string->{$i}})>1){
			 my $coverage=0; 
			 foreach my $j (@AG){
				 if ( defined $string->{$i}{$j} ) {
					 $coverage+=$string->{$i}{$j};
				 }
			 }			 
			 $l_cv->{$i}=$string->{$i}{$add}/$coverage;
			 if($l_cv->{$i}>.9){
				 foreach my $pass (keys %{$whoshere->{$i}}){
					  if($whoshere->{$i}{$pass} eq $add){
					  }else{
						  $bad_pass->{$pass}++;
					  }
				 }
			 }else{
				 foreach my $pass (keys %{$whoshere->{$i}}){
					 $bad_pass->{$pass}++;
				 }					 		 
			 }
			 if($l_cv->{$i}<.7){
				 if($i <= ($dus+19)){
					if(($i-$dus+1)>$l_kill){
					   $l_kill=$i-$dus+1; 
					}
				 }
				 if($i >= ($due-19)){
					if(($due-$i+1)>$r_kill){
					   $r_kill=$due-$i+1;  
					}
				 } 
			 }
		 }else{
			 $l_cv->{$i}=1;
			 if($string->{$i}{$add}>2){
			 }else{
				 if($i <= ($dus+19)){
					if(($i-$dus+1)>$l_kill){
					   $l_kill=$i-$dus+1; 
					}
				 }
				 if($i >= ($due-19)){
					if(($due-$i+1)>$r_kill){
					   $r_kill=$due-$i+1;  
					}
				 }
			 }
		 }
	 }else{
		 #print "\ndefined nist  i $i node $node\n"; 
	 }
  }	  		  
  if($L>($contig_size_cutoff+$l_kill+$r_kill)){ 
	  for (my $i=$dus; $i <= ($dus+$l_kill-1) ; $i++){
		  foreach my $pass (keys %{$whoshere->{$i}}){  
			 delete $track->{$pass};
		  }	
	  }
	  for (my $i=($due-$r_kill+1); $i <= $due ; $i++){
		  foreach my $pass (keys %{$whoshere->{$i}}){  
			 delete $track->{$pass};
		  }	
	  }  
	  $contig=substr($contig,$l_kill); $contig=substr($contig,0,(length($contig)-$r_kill)); 
	  $new_cov-=$r_kill; $new_cov-=$l_kill;
  }else{
	  $l_kill=0; $r_kill=0;
  }
  $L=length($contig);
  if($L>($contig_size_cutoff)){
	  $new_cov/=$L;
	  printf TIG ">contig%i|size%i|cov%.2f\n%s\n", ($node,$L,$new_cov,$contig); 
	  $tig_info->{$node}{'cov'}=.01* int(100*$new_cov);
	  $tig_info->{$node}{'length'}=$L;
	  $move+=$l_kill;
	  ($track_all,$black_pass)= &trackReads($track,$track_all,$bad_pass,$move,$black_pass); 
	  foreach my $i (keys %{$l_cv}){
		  $tig_info->{$node}{'Cvalue'}{$i-$move}=$l_cv->{$i};
	  }
  }else{
	  #print "too short node $node L $L\n"; 
	  $node--;  
  }
  undef $string; undef $whoshere, undef $bad_pass; undef $track;        
}
$track_all = &Filtertrack($track_all, $black_pass); 
$date = `date`; chomp($date); print "\nContig construction finished $date\n\n"; 
#--------------------------------  
my @aray_ins = Empiricalsize($pairead_file, $track_all, $forbiden_link, $tig_info, $node);  

($Ori_Dis, $selfpair, $tig_info)= OrientationDistance($pairead_file, $track_all, $forbiden_link, $tig_info, $node, @aray_ins);  

store \%{$Ori_Dis}, "$address"."/orientdistinfo". $base_name;   undef $Ori_Dis;

$date = `date`; chomp($date); print "Translation starteded $date\n"; 
Translate($track_all, $node, $tig_info); 
$date = `date`; chomp($date); print "Translation ended $date\n"; 
#--------------------------------  
#print "du_badpas $du_badpas du_badpas2 $du_badpas2 du_badpas3 $du_badpas3 du_badpas4 $du_badpas4 du_badpas5 $du_badpas5 du_badpas6 $du_badpas6 du_badpas7 $du_badpas7\n";
#print "k1 $kharab1 k2 $kharab2 k3 $kharab3 k4 $kharab4 selfpair $selfpair\n";

open (SF, ">$address"."/badselfpair_c$copy_li"."_e$ign_empi"."$base_name".".txt") or die "$!";   
foreach my $a (keys %$tig_info){
	if($tig_info->{$a}{'in_consis'}>20){
	   print SF  "contig $a number of badpair $tig_info->{$a}{'in_consis'}\n";
	}
}
close SF;
	
$tig_info->{1}{'seq_l_u'}=$s_l_u;
$tig_info->{1}{'seq_l_d'}=$s_l_d;

my @lengths; my $sum;
foreach my $a (keys %$tig_info){
     push @lengths, $tig_info->{$a}{'length'} ;
     $sum+=$tig_info->{$a}{'length'} ;
}	
@lengths=sort {$b <=> $a} @lengths;  

my $larges=0;my $N50;

SBASSE: for (my $i=0; $i < scalar(@lengths); $i++){        
      $larges+=$lengths[$i];
      if($larges>($sum/2)){
            $N50=$lengths[$i];
            last SBASSE;
      }
}

store \%{$tig_info}, "$address"."/div/tig_info". $base_name; 

open (CDIS, ">$address"."/div/coverage_distribution.txt");

foreach my $tig ( keys %{$tig_info}){  
	  print CDIS "$tig_info->{$tig}{'cov'} ";
}

close CDIS;

#------------------------------------
my $c_n;
open(IN,$postprocess_file) || die "Can't open $postprocess_file -- fatal\n";
while(my $line=<IN>){
   if($line=~/^=NODE.*$/){
   
   }elsif($line=~/^\>/){    
  
       my $read=<IN>; chomp $read;
       
       if($read =~/^[ACGT][0-3]([0-3]*)$/){
		   my $read =$1;
		   if ( defined $c_n->{$read} ){
		        $c_n->{$read}++;
		   }elsif ( defined $c_n->{reverse($read)} ){
		        $c_n->{reverse($read)}++;
		   }else{
		        $c_n->{$read}++;
		   }
 
       }else{
           print "format?\n";
       }
   }
}
close IN; 
open (CN, ">$address"."/div/copynumber.txt" ) or die "$!";
foreach my $r ( keys %{$c_n} ){
     print CN "$c_n->{$r} ";
}
close CN; undef $c_n;

$date = `date`; chomp($date); print "\nFinished contig building $date        N50 $N50       sum $sum\n\n";         

exit;

#------------------------------------
sub trackReads{
   my ($track, $track_all, $bad_pass, $move,$black_pass) = @_;  
   foreach my $rd (keys %{$track}){
      if($track->{$rd}{'count'}<$copy_li){ 
		  if(! defined $track_all->{$rd}{'tig'}){
			 $track_all->{$rd}{'tig'} = $track->{$rd}{'tig'};
			 $track_all->{$rd}{'start'} = $track->{$rd}{'start'}-$move;
			 $track_all->{$rd}{'end'} = $track->{$rd}{'end'}-$move;
			 $track_all->{$rd}{'count'}= $track->{$rd}{'count'};
			 delete $track->{$rd};
			 if(defined $bad_pass->{$rd}){
				 $track_all->{$rd}{'onlyfb'}++;
			 }
		  }else{
			 if(($track->{$rd}{'tig'}!=$track_all->{$rd}{'tig'}) || ($track->{$rd}{'start'}!=$track_all->{$rd}{'start'}) || ($track->{$rd}{'end'}!=$track_all->{$rd}{'end'})){
				 $du_badpas3++;
				 $black_pass->{$rd}++;
			 }else{
			     $track_all->{$rd}{'count'}++;
			 }
		  }
	  }else{
	      $du_badpas2++;
	      $black_pass->{$rd}++;
	  }
   }
   return $track_all, $black_pass;
}
#------------------------------------
sub Filtertrack{
   my ($track_all, $black_pass) = @_;  
   foreach my $rd (keys %{$track_all}){
       my $count=$track_all->{$rd}{'count'};
       my $rv_rd=reverseComplement($rd);
       if (defined $track_all->{$rv_rd}){
            if(($track_all->{$rv_rd}{'tig'}!=$track_all->{$rd}{'tig'}) || ($track_all->{$rv_rd}{'start'}!=$track_all->{$rd}{'end'}) || ($track_all->{$rv_rd}{'end'}!=$track_all->{$rd}{'start'})){
                 $black_pass->{$rd}++;
                 $black_pass->{$rv_rd}++;
                 $du_badpas6++; # print "tg1 $track_all->{$rv_rd}{'tig'} tg2 $track_all->{$rd}{'tig'} s1 $track_all->{$rv_rd}{'start'} e2 $track_all->{$rd}{'end'} e1 $track_all->{$rv_rd}{'end'} s2 $track_all->{$rd}{'start'}\n";
            }else{
                 $count+=$track_all->{$rv_rd}{'count'};
                 $du_badpas7++;
            }
       }
       if($count < $copy_li){
       
       }else{
            $black_pass->{$rd}++;
            $du_badpas4++;
       }
   }
   foreach my $rd (keys %{$black_pass}){
       delete $track_all->{$rd};
       $du_badpas5++;
   }
   return $track_all;
}
#-------------------------------- 
sub updatelocaltrack{
   my ($track, $pass, $shoroo, $tig_num, $bad_pass) = @_;
   if ($shoroo>=0){
	  $shoroo++;
	  if (!defined $bad_pass->{$pass}){
		  if(!defined $track->{$pass}){
			  $track->{$pass}{'tig'}=$tig_num;
			  $track->{$pass}{'start'}=$shoroo;
			  $track->{$pass}{'end'}=$shoroo+length($pass)-1;
			  $track->{$pass}{'count'}++;
		  }else{
			  if(($track->{$pass}{'tig'}!=$tig_num) || ($track->{$pass}{'start'}!=$shoroo) || ($track->{$pass}{'end'}!=($shoroo+length($pass)-1))){
				   $du_badpas++;
				   delete $track->{$pass};
				   $bad_pass->{$pass}++;
			  }else{
				   $track->{$pass}{'count'}++;
			  }
		  }
      }else{
		  $bad_pass->{$pass}++;
      }
   }else{
	  $shoroo*=-1;
	  if (!defined $bad_pass->{$pass}){
		  if(!defined $track->{$pass}){
			  $track->{$pass}{'tig'}=$tig_num;
			  $track->{$pass}{'start'}=$shoroo;
			  $track->{$pass}{'end'}=$shoroo-length($pass)+1;
			  $track->{$pass}{'count'}++;
		  }else{
		      if(($track->{$pass}{'tig'}!=$tig_num) || ($track->{$pass}{'start'}!=$shoroo) || ($track->{$pass}{'end'}!=($shoroo-length($pass)+1))){
		           $du_badpas++;
		           delete $track->{$pass};
		           $bad_pass->{$pass}++;
			  }else{
				   $track->{$pass}{'count'}++;
			  }
		  }
      }else{
		  $bad_pass->{$pass}++;
      }            
   }  
   return $track, $bad_pass;
}   
#--------------------------------
sub updatestring{
   my ($reshte, $short_read, $shoroo, $kinjast) = @_;  
    
   my @slice_read=split(//,$short_read);
  
   if ($shoroo>=0){
	  $shoroo++;
	  for (my $i=0; $i < scalar(@slice_read) ; $i++){
	     $reshte->{$shoroo+$i}{$slice_read[$i]}++; 
	     $kinjast->{$shoroo+$i}{$short_read}=$slice_read[$i];
	  }
   }else{
	  $shoroo*=-1;
	  for (my $i=0; $i < scalar(@slice_read) ; $i++){
		 $reshte->{$shoroo-$i}{$slice_read[$i]}++; 
		 $kinjast->{$shoroo-$i}{$short_read}=$slice_read[$i];
	  }	 
   }  
   return $reshte, $kinjast;
}  
#----------------------------------------------
sub  Empiricalsize{    
   my ($file, $track, $forbiden, $info, $mtig) = @_;
   my @aray_in; my @sizes; my $num_self; my $insert_size; my $dev_1; my $dev_2; my $mm=0; my $min_n=200; my $assoc;
   my $mean=0; my $N=0;
   foreach my $tig (keys %{$info}){  
        $mean+=$info->{$tig}{'cov'};  $N++; $info->{$tig}{'in_consis'}=0;
   }
   
   open(IN,$file) || die "Can't open $file -- fatal\n"; 
   AKHAREMATE: while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
          if ($mm==1) {          
		      print "Associated library: $assoc\n";
			  if ( scalar(@sizes) > 0){
					  @sizes= sort @sizes;   
					  my $L=scalar(@sizes);    

					 my $sum=0;
					 foreach my $t (@sizes){
						   $sum+=$t; 
					  }
					  my $new_i=int ($sum/$L)+1;
					  if ( $L > $min_n ){
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
							 push @aray_in, $new_i;     
					  }else{
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
							 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
							 push @aray_in, $insert_size ;    
					  }
					  
			  }else{
    	  	   	    print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
					push @aray_in, $insert_size ; 
              }
          }
          $mm=1;
          $insert_size=$2; $assoc=$1;

          my $assoc_trim=$1;
          
          if($assoc_trim =~ /(.*)\/([^\/]*)$/){
          	 $assoc_trim=$2;
          
          }else{
          
          } 
          
          open (DDIS, ">$address"."/div/insertsize_distribution_$assoc_trim"."_$insert_size".".txt");
          
          undef @sizes;
          if($insert_size>5000){
               $dev_1=1.1; $dev_2=2;
          }elsif($insert_size>1000){
               $dev_1=1.3; $dev_2=2.5;
          }elsif($insert_size>500){
               $dev_1=1.6; $dev_2=3.25;
          }elsif($insert_size>250){
               $dev_1=1.6; $dev_2=3.75;
          }else{
               $dev_1=1.6; $dev_2=4;
          }        
          #print "insert_size $insert_size dev_1 $dev_1 dev_2 $dev_2\n";          
      }elsif($line =~ /^Fragment$/){
          if ($mm==1) {          
		      print "Associated library: $assoc\n";
			  if ( scalar(@sizes) > 0){
					  @sizes= sort @sizes;   
					  my $L=scalar(@sizes);    

					 my $sum=0;
					 foreach my $t (@sizes){
						   $sum+=$t; 
					  }
					  my $new_i=int ($sum/$L)+1;
					  if ( $L > $min_n ){
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
							 push @aray_in, $new_i;     
					  }else{
							 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
							 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
							 push @aray_in, $insert_size ;    
					  }
			  }else{
					print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
					push @aray_in, $insert_size ; 
              }
          }
          $mm=0;          
          last AKHAREMATE;     
      }elsif($line =~ /^([^\>]*)$/i){  
      
         my $sdna = $1;
         if($sdna =~/^([ACGT])([0-3]*)\:([ACGT])([0-3]*)$/i){
            my $read_a=substr($4,1); my $read_b=substr($2,1); 

            if((defined $track->{$read_a}) && (defined $track->{$read_b})){### both pairs assembled	
                  if((!defined $track_all->{$read_a}{'onlyfb'}) && (!defined $track_all->{$read_b}{'onlyfb'})){		
					 if((!defined $track->{$read_a}{'tig'}) || (!defined $track->{$read_b}{'tig'})){
						print "unexpected problem\n"; exit;
					 } 
					 my $tig_a = $track->{$read_a}{'tig'};
					 my $tig_b = $track->{$read_b}{'tig'};    
					 my $A_length = $info->{$tig_a}{'length'};
					 my $A_start = $track->{$read_a}{'start'};
					 my $A_end = $track->{$read_a}{'end'};
					 my $B_length = $info->{$tig_b}{'length'};
					 my $B_start = $track->{$read_b}{'start'} ;
					 my $B_end = $track->{$read_b}{'end'};
					 #if(($A_length>100) && ($B_length>100)){		  
						 if($tig_a != $tig_b){####paired reads located on <> contigs
						 }else{###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
					         if( $A_length > (2*$insert_size) ){				  
								  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
										if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'} ){#  -> -> 	      
											  my $imp= $B_start-$A_start; print DDIS "$imp ";
											  my $d=$B_start-$A_start-$insert_size;	
											  
											  if ( (abs($d)<($dev_2*$insert_size)) && ($imp>0)){
												   push @sizes, $imp; 
											  }else{
												   
											  }
										}else{     
										   
										}
								  
								  }else{
									  if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'}){#  <-  <-
											  my $imp= $A_start-$B_start; print DDIS "$imp ";
											  my $d=$A_start-$B_start-$insert_size;									  
	
											  if ( (abs($d)<($dev_2*$insert_size)) && ($imp>0)){
												   push @sizes, $imp;  
											  }else{
												   
											  }
									  
									  }else{     
										   
									  }
									  
								  }	  
							 }	  
						 } 
					 #} 
				  }	 
		    }     
         }
      }
   }
   if ($mm==1) {          
		  print "Associated library: $assoc\n";
		  if ( scalar(@sizes) > 0){
				  @sizes= sort @sizes;   
				  my $L=scalar(@sizes);    

				 my $sum=0;
				 foreach my $t (@sizes){
					   $sum+=$t; 
				  }
				  my $new_i=int ($sum/$L)+1;
				  if ( $L > $min_n ){
						 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs)\n\n";
						 push @aray_in, $new_i;     
				  }else{
						 print "Suggested insert_size: $insert_size         Emperical insert_size: $new_i   (based on $L pairs , minimum required $min_n pairs)\n";
						 print "Not enough data points, keeping the suggested insert_size: $insert_size\n\n";
						 push @aray_in, $insert_size ;    
				  }
				  
		  }else{
				 print "No data point to make emperical estimation of insert_size (no pair located on the same contig), keeping the suggested insert_size: $insert_size\n\n";
				 push @aray_in, $insert_size ; 
          }
   }      
   close IN; close DDIS;  print "\n";
   return @aray_in;
}
#----------------------------------------------
sub  OrientationDistance{    
   my ($file, $track, $forbiden, $info, $mtig, @aray_in) = @_;
   my $ORI_DIS; my $num_self; my $insert_size; my $dev_1; my $dev_2;
   my $mean=0; my $N=0;
   foreach my $tig (keys %{$info}){  
        $mean+=$info->{$tig}{'cov'};  $N++; $info->{$tig}{'in_consis'}=0;
   }
#    $mean=int($mean/$N); my $cut=2*$mean; print "for ori-dis nmtig $mtig mean_cov $mean cut $cut\n";
#    for(my $tig=1;$tig<$mtig;$tig++){
#        if(($info->{$tig}{'length'}>(2*$insert_size)) && ($info->{$tig}{'cov'}<$cut) ){
#            my $tot_window=int($info->{$tig}{'length'}/$seq_l);
#            $info->{$tig}{'tot_w'}=$tot_window; 
#            for(my $i=1;$i<=$info->{$tig}{'tot_w'};$i++){
#                $info->{$tig}{'pilar'}{$i}=0; 
#            }
#        }
#    }
   
   open(IN,$file) || die "Can't open $file -- fatal\n";
   AKHAREMATE: while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
           if ($ign_empi==1){
           		$insert_size = $2;
           }else{
		        $insert_size = shift(@aray_in);   
           }
		   print "Associated library: $1\n";
           print "Suggested insert_size: $2         We will use: $insert_size\n\n";
          if($insert_size>5000){
               $dev_1=1.1; $dev_2=.2;
          }elsif($insert_size>1000){
               $dev_1=1.3; $dev_2=.5;
          }else{
               $dev_1=1.6; $dev_2=1;
          }          
          #print "insert_size $insert_size dev_1 $dev_1 dev_2 $dev_2\n";          
      }elsif($line =~ /^Fragment$/){
          last AKHAREMATE;     
      }elsif($line =~ /^([^\>]*)$/i){
         my $sdna = $1;
         if($sdna =~/^([ACGT])([0-3]*)\:([ACGT])([0-3]*)$/i){
            my $read_a=substr($4,1); my $read_b=substr($2,1); 
            
            if((defined $track->{$read_a}) && (defined $track->{$read_b})){### both pairs assembled
                if((!defined $track_all->{$read_a}{'onlyfb'}) && (!defined $track_all->{$read_b}{'onlyfb'})){				     
					 if((!defined $track->{$read_a}{'tig'}) || (!defined $track->{$read_b}{'tig'})){
						print "unexpected problem\n"; exit;
					 }
					 my $tig_a = $track->{$read_a}{'tig'};
					 my $tig_b = $track->{$read_b}{'tig'};    
					 my $A_length = $info->{$tig_a}{'length'};
					 my $A_start = $track->{$read_a}{'start'};
					 my $A_end = $track->{$read_a}{'end'};
					 my $B_length = $info->{$tig_b}{'length'};
					 my $B_start = $track->{$read_b}{'start'} ;
					 my $B_end = $track->{$read_b}{'end'};
					 #if(($A_length>100) && ($B_length>100)){		  
					     if($tig_a != $tig_b){####paired reads located on <> contigs
							if(! defined $forbiden->{$tig_a}{$tig_b}){
								if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
									  if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'} ){#  -> -> 
										 my $d=$insert_size+$A_start-$B_start;
						  
										 my $gap=$d-$info->{$tig_a}{'length'};
										 if(abs($gap)<($dev_1*$insert_size)){
												 if ($tig_a<$tig_b){
													$ORI_DIS->{$tig_a}{$tig_b}{'P'}++;
													$ORI_DIS->{$tig_a}{$tig_b}{'PD'}+=$d;
												 }else{    
													$ORI_DIS->{$tig_b}{$tig_a}{'P'}++;
													$ORI_DIS->{$tig_b}{$tig_a}{'PD'}-=$d;
												 }									 
										  }else{
											 $kharab1++;
										  }
									  }else{# -> <-
										 my $d=$insert_size+$A_start+$B_start;
																 
										 my $gap=$d-$info->{$tig_a}{'length'}-$info->{$tig_b}{'length'};
										 if(abs($gap)<($dev_1*$insert_size)){
												 if ($tig_a<$tig_b){
													$ORI_DIS->{$tig_a}{$tig_b}{'N'}++;
													$ORI_DIS->{$tig_a}{$tig_b}{'ND'}+=$d;
												 }else{
													$ORI_DIS->{$tig_b}{$tig_a}{'N'}++;
													$ORI_DIS->{$tig_b}{$tig_a}{'ND'}+=$d;
												 }
										 }else{                             
											 $kharab2++;
										 }
									  }
								}else{
									  if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'}){#  <-  <-
										 my $d=-($insert_size-$A_start+$B_start);	
						  
										 my $gap=-$d-$info->{$tig_b}{'length'};
										 if(abs($gap)<($dev_1*$insert_size)){
												 if ($tig_a<$tig_b){
													$ORI_DIS->{$tig_a}{$tig_b}{'P'}++;
													$ORI_DIS->{$tig_a}{$tig_b}{'PD'}+=$d;
												 }else{
													$ORI_DIS->{$tig_b}{$tig_a}{'P'}++;
													$ORI_DIS->{$tig_b}{$tig_a}{'PD'}-=$d;
												 }
										 }else{							 
											 $kharab3++;
										 }
									  }else{    #   <- ->  
										 my $d=-($insert_size-$A_start-$B_start);
										 
										 my $gap=-$d;
										 if(abs($gap)<($dev_1*$insert_size)){
												 if ($tig_a<$tig_b){
													$ORI_DIS->{$tig_a}{$tig_b}{'N'}++;
													$ORI_DIS->{$tig_a}{$tig_b}{'ND'}+=$d;
												 }else{
													$ORI_DIS->{$tig_b}{$tig_a}{'N'}++;
													$ORI_DIS->{$tig_b}{$tig_a}{'ND'}+=$d;
												 }
										 }else{						 
											 $kharab4++
										 }									 
									  }
								}
							}
						 }else{###Clone, paired reads located on the same contig -- could be used to investigate misassemblies
							  if ($track->{$read_a}{'start'} < $track->{$read_a}{'end'}){
									if ($track->{$read_b}{'start'} < $track->{$read_b}{'end'} ){#  -> -> 
										  my $d=$B_start-$A_start-$insert_size;
										  if (abs($d)<($dev_2*$insert_size)){
											   $num_self++;
											   if(defined $info->{$tig_a}{'tot_w'}){ 
												   my $w_s; my $w_e;
												   my $dus=1+int($track->{$read_a}{'start'}/$seq_l);
												   if($dus>$info->{$tig_a}{'tot_w'}){  
													  $w_s=$info->{$tig_a}{'tot_w'}; 
												   }else{
													  $w_s=$dus;
												   }
												   undef $dus; my $due=1+int($track->{$read_b}{'end'}/$seq_l);
												   if($due>$info->{$tig_a}{'tot_w'}){  
													   $w_e=$info->{$tig_a}{'tot_w'};
												   }else{
													  $w_e=$due;
												   } 
												   for(my $i=$w_s;$i<=$w_e;$i++){
														$info->{$tig_a}{'pilar'}{$i}++; 
												   }
											   }
										  }else{
											  $info->{$tig_a}{'in_consis'}++; #  print "tig $tig_a   d $d   B $B_start A $A_start $insert_size\n\n";
										  }
									}else{     
									   $info->{$tig_a}{'in_consis'}++; #print "ndashtim 0 $tig_a $tig_b $read_a $read_b\n\n\n";
									}
							  
							  }else{
								  if ($track->{$read_b}{'start'} > $track->{$read_b}{'end'}){#  <-  <-
										  my $d=$A_start-$B_start-$insert_size;
										  if (abs($d)<($dev_2*$insert_size)){
											   $num_self++;
											   if(defined $info->{$tig_a}{'tot_w'}){ 
												   my $w_s; my $w_e;
												   my $dus=1+int($track->{$read_b}{'end'}/$seq_l);
												   if($dus>$info->{$tig_a}{'tot_w'}){  
													  $w_s=$info->{$tig_a}{'tot_w'}; 
												   }else{
													  $w_s=$dus;
												   }
												   undef $dus; my $due=1+int($track->{$read_a}{'start'}/$seq_l);
												   if($due>$info->{$tig_a}{'tot_w'}){  
													  $w_e=$info->{$tig_a}{'tot_w'};
												   }else{
													  $w_e=$due;
												   } 
												   for(my $i=$w_s;$i<=$w_e;$i++){
														$info->{$tig_a}{'pilar'}{$i}++; 
												   }
											   }	   
										  }else{
											  $info->{$tig_a}{'in_consis'}++;  
										  }
								  
								  }else{     
									   $info->{$tig_a}{'in_consis'}++; 
								  }
								  
							  }	  
						 } 
				     #}		 
				}	 
		    }     
         }
      }
   }
   close IN; print "\n";
   return $ORI_DIS, $num_self, $info;
}
#----------------------------------------------
sub Translate{
    my ($track_a, $max_tigcount, $info) = @_;
    
    my $fi_contig="$address"."/contigs_colorspace". $base_name .".fasta" ;
    
	my $Base2Color;  
	$Base2Color->{'A'}{'A'}=0;
	$Base2Color->{'A'}{'T'}=3;
	$Base2Color->{'A'}{'C'}=1;
	$Base2Color->{'A'}{'G'}=2;
	$Base2Color->{'T'}{'A'}=3;
	$Base2Color->{'T'}{'T'}=0;
	$Base2Color->{'T'}{'C'}=2;
	$Base2Color->{'T'}{'G'}=1;
	$Base2Color->{'C'}{'A'}=1;
	$Base2Color->{'C'}{'T'}=2;
	$Base2Color->{'C'}{'C'}=0;
	$Base2Color->{'C'}{'G'}=3;
	$Base2Color->{'G'}{'A'}=2;
	$Base2Color->{'G'}{'T'}=1;
	$Base2Color->{'G'}{'C'}=3;
	$Base2Color->{'G'}{'G'}=0;

	my @con_interval; my $sum=0;
	open (NTIG, $fi_contig) or die "$!";
	while (<NTIG>){
	   my $tig;
	   if ($_=~ /^\>/){
			my @string = split(/\|/,$_);
			$tig=substr($string[0],7);
			my $length=substr($string[1],4);
			$sum+=$length;
	   }     
	   if($sum>500000000){
		   $sum=0;
		   push @con_interval, $tig;
	   }
	}
	close NTIG; undef $sum; push @con_interval, $max_tigcount;
	  
	open (NTIG, $fi_contig) or die "$!"; 
			 
	my $betha=.5; my $alpha=6;
	my $Ps=exp($betha); my $Pc=exp($alpha);
	
	my @V03 = (0 .. 3);
	my @AG = ('A','T','C','G');
	# ---------------------------------------- 
	my $current_i=0; my $fbase = firstBase($pairead_file, $track_a, 1, $con_interval[0]);  my $max_c=$con_interval[0];

	open (OUT, ">$address"."/contigs". $base_name .".fasta" ) or die "$!";   my $tot_du=0; 
	
	while (<NTIG>){  
	   my @Col; my $tig;
	   if ($_=~ /^\>/){
			my @string = split(/\|/,$_);
			$tig=substr($string[0],7);
			my $cont=<NTIG>;
			chomp($cont);
			@Col= split(//,$cont); $tot_du+=scalar(@Col);
	   }else{
		  print "unexpected problem\n"; exit;
	   }     

	  if ( $tig % 500 == 0 ){
			my $date = `date`; chomp($date); print "contig $tig $date\n";
	  }
	  
	   if($tig>$max_c){
		   my $min_c=shift(@con_interval); $max_c=$con_interval[0]; 
		   print  "contig $tig\n"; 
		   $tot_du=0;
		   $fbase = firstBase($pairead_file, $track_a, $min_c, $max_c);  
		   my $date = `date`; chomp($date); print "$date\n";     
	   }
	   foreach my $pos (keys %{$fbase->{$tig}}){
			 my $sum=0;
			 foreach my $j (keys %{$fbase->{$tig}{$pos}}){
				  $sum+=$fbase->{$tig}{$pos}{$j}; 
			 }
			 if($sum>20){
				 foreach my $j (keys %{$fbase->{$tig}{$pos}}){
					  $fbase->{$tig}{$pos}{$j}=20*$fbase->{$tig}{$pos}{$j}/$sum; 
				 }
				 my $sum=0;
				 foreach my $j (keys %{$fbase->{$tig}{$pos}}){
					  $sum+=$fbase->{$tig}{$pos}{$j}; 
				 } 
				 $sum=int(($sum+.000001));
			 }	 
	   }
	   my $Seq; my $Score; my $TotS=0;
	#    $Seq->{.5}{'A'};
	#    $Seq->{.5}{'T'};
	#    $Seq->{.5}{'C'};
	#    $Seq->{.5}{'G'};
	   if(defined $fbase->{$tig}{.5}){   
		   if(defined $fbase->{$tig}{.5}{'A'}){
			   $Score->{'A'}=$Ps**$fbase->{$tig}{.5}{'A'}; $TotS+=$fbase->{$tig}{.5}{'A'};
		   }else{
			   $Score->{'A'}=1;
		   }
		   if(defined $fbase->{$tig}{.5}{'T'}){
			   $Score->{'T'}=$Ps**$fbase->{$tig}{.5}{'T'}; $TotS+=$fbase->{$tig}{.5}{'T'};
		   }else{
			   $Score->{'T'}=1;
		   }  
		   if(defined $fbase->{$tig}{.5}{'C'}){
			   $Score->{'C'}=$Ps**$fbase->{$tig}{.5}{'C'}; $TotS+=$fbase->{$tig}{.5}{'C'};
		   }else{
			   $Score->{'C'}=1;
		   }
		   if(defined $fbase->{$tig}{.5}{'G'}){
			   $Score->{'G'}=$Ps**$fbase->{$tig}{.5}{'G'}; $TotS+=$fbase->{$tig}{.5}{'G'};
		   }else{
			   $Score->{'G'}=1;
		   }      
	   }else{
		   $Score->{'A'}=1;    
		   $Score->{'T'}=1;    
		   $Score->{'C'}=1;      
		   $Score->{'G'}=1;     
	   }
	
	  my $x=0;
	   foreach my $du (keys %{$Score}){
		   $x += $Score->{$du}**2
	   }
	   $x=$x**.5;
	   foreach my $du (keys %{$Score}){
		   $Score->{$du}/=$x;
	   }
	   $x=0;
	   foreach my $du (keys %{$Score}){
		   $x += $Score->{$du}**2;
	   }
	   $x=int(($x+.000001));
	   if ($x!=1){
			   print "e $x\n";
			print "inja 342\n";
	   } 
	   for (my $pos_fb= 1.5; $pos_fb < (scalar(@Col)+1); $pos_fb++){  

			my $jj=0;
			my $TempSeq; my $TScore; 
			foreach my $j (@AG){
			   my $TempScore;
			   foreach my $z (@AG){
				  $TempScore->{$z}=$Score->{$z}*($Pc**($info->{$tig}{'Cvalue'}{($pos_fb-.5)}*($Base2Color->{$z}{$j} == $Col[($pos_fb-1.5)]))); 
			   } 
			   foreach my $z (sort { $TempScore->{$b} <=> $TempScore->{$a} } keys %{$TempScore}){  
					  if(defined $fbase->{$tig}{$pos_fb}){
						  if(defined $fbase->{$tig}{$pos_fb}{$j}){
							 $TScore->{$j}=$TempScore->{$z}*($Ps**$fbase->{$tig}{$pos_fb}{$j});
							 $TotS+=$fbase->{$tig}{$pos_fb}{$j};  
						  }else{
							 $TScore->{$j}=$TempScore->{$z};
						  }
					  }else{
						  $TScore->{$j}=$TempScore->{$z};  
					  } 
					  $Seq->{$pos_fb}{$j}=$z;
					  last;
			   } 
			}
			foreach my $j (@AG){
			  $Score->{$j}=$TScore->{$j};
			}   
			my $x=0;
			foreach my $du (keys %{$Score}){
				$x += $Score->{$du}**2;
			}
			$x=$x**.5;
			foreach my $du (keys %{$Score}){
				$Score->{$du}/=$x;
			}
	   }   
	   if ($TotS!=0){
		   if ($TotS<10){
			  
		   }
		   my $Fin_s; my $curent;
		   foreach my $z (sort { $Score->{$b} <=> $Score->{$a} } keys %{$Score}){  
				  $Fin_s=$z;
				  $curent=$z;
				  last;
		   }
		   for (my $pos_fb=scalar(@Col)+.5; $pos_fb >1 ; $pos_fb--) {  
				   $curent=$Seq->{$pos_fb}{$curent}; 
				   $Fin_s=$curent.$Fin_s; 
		   }        
		   printf OUT ">contig%i|size%i|cov%i\n%s\n", ($tig,length($Fin_s),$info->{$tig}{'cov'},$Fin_s); 

# 		   my $disagree;
# 		   foreach my $z (sort { $Score->{$b} <=> $Score->{$a} } keys %{$Score}){  
# 				  $curent=$z;
# 				  last;
# 		   }  
# 		   for (my $pos_fb=scalar(@Col)+.5; $pos_fb >1 ; $pos_fb--) {  
# 				if($Base2Color->{$Seq->{$pos_fb}{$curent}}{$curent} != $Col[($pos_fb-1.5)]){
# 					  print "tig $tig L $info->{$tig}{'length'} dup $info->{$tig}{'dup'} pos $pos_fb $Seq->{$pos_fb}{$curent} $curent $Col[($pos_fb-1.5)]\n"; 
# 					  print "a $fbase->{$tig}{$pos_fb-1}{'A'} t $fbase->{$tig}{$pos_fb-1}{'T'} c $fbase->{$tig}{$pos_fb-1}{'C'} g $fbase->{$tig}{$pos_fb-1}{'G'} - a $fbase->{$tig}{$pos_fb}{'A'} t $fbase->{$tig}{$pos_fb}{'T'} c $fbase->{$tig}{$pos_fb}{'C'} g $fbase->{$tig}{$pos_fb}{'G'}\n\n";
# 					  $disagree++;
# 				}
# 				$curent=$Seq->{$pos_fb}{$curent}; 
# 		   }
# 		   if($disagree>0){
# 				#print "disagree $disagree\n\n";  
# 		   }              
 	   }else{
  		   #print "tig $tig length $Ld Totfb $TotS\n";
  		   $info->{$tig}{'translation'}=0;
 	   }
	   delete $fbase->{$tig};
	}
	close NTIG; close OUT; 
}	
# ---------------------------------------- 
sub firstBase{    
   my ($file, $track, $minc, $maxc) = @_;
   my $fb; my $du=0; my $du1=0;
   my %BaseColor2Base=(
						 A=>['A', 'C', 'G', 'T'],
						 C=>['C', 'A', 'T', 'G'],
						 G=>['G', 'T', 'A', 'C'],
						 T=>['T', 'G', 'C', 'A']
						 );
   
   my ($mate,$frag)=(0,0);  
   open(IN,$file) || die "Can't open $file -- fatal\n";
   AKHARESH: while(my $line=<IN>){
      chomp($line);
      if($line =~ /^Mate.(.*)_(\d*)$/){
          $mate=1; $frag=0;
      }elsif($line =~ /^Fragment$/){
          $mate=0; $frag=1; 
      }elsif($line =~ /^([^\>]*)$/i){
         my $sdna = $1; $du++;
         if($mate == 1){
				 if($sdna =~/^([ACGT])([0-3]*)\:([ACGT])([0-3]*)$/i){
					#my $read_a = $2;
					#my $read_b = $4;
					my $read_a=substr($4,1); my $read_b=substr($2,1);  
					#my $read_a = $4;
					#my $read_b = $2;
					if(defined $track->{$read_a}){
                        if(!defined $track->{$read_a}{'tig'}){
                            print "yo yo 2"; exit;
                        }					    
						my $tig_a = $track->{$read_a}{'tig'};  
						if($tig_a>=$minc && $tig_a<=$maxc){
							my $A_start = $track->{$read_a}{'start'}; $du1++;
							my $A_end = $track->{$read_a}{'end'};
							if($A_start<$A_end){
							   my $cfb=$BaseColor2Base{$3}[substr($4,0,1)];
							   $fb->{$tig_a}{$A_start-0.5}{$cfb}++;
							}else{
							   my $cfb=$BaseColor2Base{$3}[substr($4,0,1)];
							   $cfb=~tr/ACTG/TGAC/;
							   $fb->{$tig_a}{$A_start+0.5}{$cfb}++;
							}
							if($tig_a>$maxc){
							   print "sss $tig_a $maxc\n";
							}
						}		
					}
					if(defined $track->{$read_b}){
                        if(!defined $track->{$read_b}{'tig'}){
                            print "yo yo 3"; exit;
                        }					
						my $tig_b = $track->{$read_b}{'tig'};  
						if($tig_b>=$minc && $tig_b<=$maxc){
							my $B_start = $track->{$read_b}{'start'}; $du1++;
							my $B_end = $track->{$read_b}{'end'};
							if($B_start<$B_end){
							   my $cfb=$BaseColor2Base{$1}[substr($2,0,1)];
							   $fb->{$tig_b}{$B_start-0.5}{$cfb}++;
							}else{
							   my $cfb=$BaseColor2Base{$1}[substr($2,0,1)];
							   $cfb=~tr/ACTG/TGAC/;
							   $fb->{$tig_b}{$B_start+0.5}{$cfb}++;
							}
						}	
					}       
				 }  
		 }else{
		         if ($sdna =~ /^([ACGT])([0-3]*)$/i){
                    my $read_a=substr($2,1);
					if(defined $track->{$read_a} ){
						if(defined $track->{$read_a}{'tig'}){
							my $tig_a = $track->{$read_a}{'tig'};  
							if($tig_a>=$minc && $tig_a<=$maxc){
								my $A_start = $track->{$read_a}{'start'}; $du1++;
								my $A_end = $track->{$read_a}{'end'};
								if($A_start<$A_end){
								   my $cfb=$BaseColor2Base{$1}[substr($2,0,1)];
								   $fb->{$tig_a}{$A_start-0.5}{$cfb}++;
								}else{
								   my $cfb=$BaseColor2Base{$3}[substr($2,0,1)];
								   $cfb=~tr/ACTG/TGAC/;
								   $fb->{$tig_a}{$A_start+0.5}{$cfb}++;
								}
								if($tig_a>$maxc){
								   print "sss $tig_a $maxc\n";
								}
							}		
						}         
                    }  
                 }
		 }
		  if (($du % 2000000)==0){
			  #print "du $du du1 $du1\n";
		  }
      }      
   } 
   foreach my $read (keys %{$track}){
       my $tig = $track->{$read}{'tig'};  
       if($tig>=$minc && $tig<$maxc){
           delete $track->{$read};
       }
   }
   close IN;
   return $fb;
}
#-----------------
sub reverseComplement{
   $_ = shift;
   $_ = uc();
   tr/ATGC/TACG/;
   return (reverse());
}
#-----------------------